/// \file Butterfly.cpp Source file for the Butterfly class.

#include "Butterfly.hpp"

SDL_Surface *Butterfly::dayButterfly;
SDL_Surface *Butterfly::nightButterfly;
Mix_Chunk *Butterfly::powerupSound;
unsigned int Butterfly::butterflycounter = 0;
unsigned int Butterfly::livebutterflycount = 0;

Butterfly::Butterfly(Level *l, Vec2f p, bool isDayFly) :
	GameObject(l, p, BUTTERFLY_DEFAULT_MAX_SPEED, BUTTERFLY_DEFAULT_RADIUS),
	isDayButterfly(isDayFly)
{
	livebutterflycount++;
	if(butterflycounter++ == 0)
	{
		dayButterfly = LoadSprite(PKGDATADIR"Artwork/objects/butterfly/butterfly_day.png",
								  1,
								  level->GetTileWidth(),
								  level->GetTileHeight());
		nightButterfly = LoadSprite(PKGDATADIR"Artwork/objects/butterfly/butterfly_night.png",
									1,
									level->GetTileWidth(),
									level->GetTileHeight());
		powerupSound = LoadSound(PKGDATADIR"Sounds/ButterflyPowerup.ogg");
	}

	UpdateImage();
}

Butterfly::~Butterfly()
{
	if (!GetDisabled()) {
		assert(livebutterflycount > 0);
		--livebutterflycount;
	}

	if(--butterflycounter == 0) {
		SDL_FreeSurface(dayButterfly);
		SDL_FreeSurface(nightButterfly);
		Mix_FreeChunk(powerupSound);
	}
}

void Butterfly::Reset()
{
	this->GameObject::Reset();

	max_speed = BUTTERFLY_DEFAULT_MAX_SPEED;
	radius = BUTTERFLY_DEFAULT_RADIUS;

	UpdateImage();
}

void Butterfly::UpdateImage()
{
	if (isDayButterfly)
		image = dayButterfly;
	else
		image = nightButterfly;
}

std::string Butterfly::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"butterfly\"";

	if (isDayButterfly)
		xml << " color=\"orange\"" << "/>\n";
	else
		xml << " color=\"violet\"" << "/>\n";

	return xml.str();
}
// vim: set ts=4 sts=4 sw=4 noet:
