/// \file Butterfly.hpp Header for the Butterfly class.

/// \brief Include guard for Butterfly
#ifndef INCLUDED_BUTTERFLY_H
#define INCLUDED_BUTTERFLY_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "SoundManager.hpp"
#include "GameObject.hpp"
#include "Utility.hpp"

/**
 * \brief Class representing a Butterfly.
 */
class Butterfly : public GameObject {
public:
	/** \brief Constructs a Butterfly in Level l at position p
	 * \param l a pointer to the Level this Butterfly is in
	 * \param p the Vec2f position of this Butterfly.
	 * \param isDayFly true if this is a day butterfly, false if it is night.
	 */
	Butterfly(Level *l, Vec2f p, bool isDayFly);

	/** \brief Destructs a Butterfly. */
	~Butterfly();

	/** \brief Runs one cycle of this Butterfly. */
	void Run() {};

	/** \brief Resets this Butterfly to its original state. */
	void Reset(void);

	inline void SetDisabled(bool d);

	/** \brief Returns true if this is a day butterfly, false if it is night.
	 * \return true if this is a day butterfly, false if it is night
	 */
	inline bool IsDayButterfly(void) const { return isDayButterfly; }

	/** \brief Gets the GameObjectType of this Butterfly.
	 * \return the GameObjectType of this Butterfly
	 */
	inline GameObjectType GetType(void) const { return GAME_OBJECT_BUTTERFLY; }

	std::string GetXMLRepresentation(const std::string& indent) const;

	void PlayCollisionSound() const;

	static unsigned int LiveButterflyCount() { return livebutterflycount; }

private:
	/** \brief Updates the image of this Butterfly.
	*/
	void UpdateImage(void);

	static SDL_Surface *dayButterfly;
	static SDL_Surface *nightButterfly;
	static Mix_Chunk *powerupSound;		///< Sound played when eaten

	static unsigned int butterflycounter;		///< Number of Butterflys objects
	static unsigned int livebutterflycount;		///< Number of live Butterflys

	bool isDayButterfly; ///< True if this is a day butterfly, false if it is night.
};

inline void Butterfly::PlayCollisionSound() const
{
	SoundManager::PlayChunk(powerupSound);
}

inline void Butterfly::SetDisabled(bool d)
{
	if (!disabled && d) {
		assert(livebutterflycount > 0);
		livebutterflycount--;
	} else if (disabled && !d) {
		livebutterflycount++;
	}

	this->GameObject::SetDisabled(d);
}

#endif /* INCLUDED_BUTTERFLY_H */
// vim: set ts=4 sts=4 sw=4 noet:
