#include <iostream>
#include <string>

#include "Utility.hpp"
#include "Button.hpp"

Mix_Chunk *Button::hoverNoise;
Mix_Chunk *Button::clickNoise;
unsigned int Button::buttoncount = 0;

/** The width of the black border around a selected Button */
static const int BUTTON_BORDER = 2;

Button::Button(int bx, int by,
			   int bheight, int bwidth,
			   std::string bmessage, std::string bpath) :
	x(bx),
	y(by),
	height(bheight),
	width(bwidth),
	message(bmessage),
	bstate(Button::BUTTON_STATE_DEFAULT)
{
	if (buttoncount++ == 0) {
		hoverNoise = LoadSound(PKGDATADIR"Sounds/ButtonSelected2.wav");
		clickNoise = LoadSound(PKGDATADIR"Sounds/ButtonSelected1.wav");
	}

	// TODO:  Scale all images, find some heuristic to do it
	if (message != "LE" && message != "LE-C") {
		image = LoadImageSDL(bpath.c_str(), bwidth, bheight*3);
	} else {
		image = LoadImageSDL(bpath.c_str());
	}
}

Button::~Button()
{
	if (--buttoncount == 0) {
		Mix_FreeChunk(hoverNoise);
		Mix_FreeChunk(clickNoise);
	}

	if (image)
		SDL_FreeSurface(image);
}

Button::Button(int bx, int by,
			   int bheight, int bwidth,
			   std::string bmessage, ButtonType btype) :
	x(bx),
	y(by),
	height(bheight),
	width(bwidth),
	message(bmessage),
	type(btype),
	bstate(Button::BUTTON_STATE_DEFAULT)
{
	if (buttoncount++ == 0) {
		hoverNoise = LoadSound(PKGDATADIR"Sounds/ButtonSelected2.wav");
		clickNoise = LoadSound(PKGDATADIR"Sounds/ButtonSelected1.wav");
	}

	color = false;
	switch(btype) {
		case GAME_OBJECT_BAT:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/bat/bat.png");
			break;
		case GAME_OBJECT_BEE:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/bee/bee-yellow.png");
			color = true;
			break;		
		case GAME_OBJECT_BUTTERFLY:
			image = LoadImageSDL(PKGDATADIR"Artwork/objects/butterfly/butterfly_day.png");
			break;
		case GAME_OBJECT_CENTIPEDE:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/rolly_polly/rolly_polly.png");
			break;
		case GAME_OBJECT_CHARLIE:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/charlie/charlie-green.png");
			color = true;
			break;
		case GAME_OBJECT_SCORPION:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/scorpion/scorpion.png");
			break;
		case GAME_OBJECT_DOOR:
			image = LoadImageSDL(PKGDATADIR"Artwork/objects/leafdoor/leafdoor.png");
			break;
		case GAME_OBJECT_FLOWER:
			image = LoadImageSDL(PKGDATADIR"Artwork/objects/flower/flower-orange.png");
			color = true;
			break;
		case GAME_OBJECT_GOAL:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/goal/goal.png");
			break;			
		case GAME_OBJECT_MILLIPEDE:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/centipede/centipede.png");
			break;
		case GAME_OBJECT_SNAKE:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/snake/snake_asleep-green.png");
			color = true;
			break;
		case GAME_OBJECT_SWITCH:
			image = LoadImageSDL(PKGDATADIR"Artwork/objects/switches/switch.png");
			break;
		case GAME_OBJECT_TIGERCUB:
			image = LoadImageSDL(PKGDATADIR"Artwork/creatures/tiger/tiger.png");
			break;
		case TILE_BRIDGE:
			image = LoadImageSDL(PKGDATADIR"Artwork/tiles/bridge/bridge_center.png");
			break;
		case TILE_LAND:
			image = LoadImageSDL(PKGDATADIR"Artwork/tiles/land/land1.png");
			color = true;
			break;
		case TILE_ROCK:
			image = LoadImageSDL(PKGDATADIR"Artwork/tiles/rock/rock.png");
			break;
		case TILE_TREE:
			image = LoadImageSDL(PKGDATADIR"Artwork/tiles/tree/tree.png");
			break;
		case TILE_WATER:
			image = LoadImageSDL(PKGDATADIR"Artwork/tiles/lake/lake_cc.png");
			break;
		default:
#ifdef DEBUG
			std::cerr << "Urecognized button type " << btype << "\n";
#endif
			image = NULL;
			break;
	}

	bstate = BUTTON_STATE_DEFAULT;
}

void Button::Draw(SDL_Surface *screen) const
{
	if (message != "LE" && message != "LE-C") {
		int frame = 0;
		switch(bstate) {
			case BUTTON_STATE_DEFAULT:
				break;
			case BUTTON_STATE_HOVER:
				frame = 1;
				break;
			case BUTTON_STATE_SELECTED:
				frame = 2;
				break;
		}
		SDL_Rect clip, offset;
		clip.x = 0;
		clip.y = frame*height;
		clip.w = width;
		clip.h = height;
		offset.x = x;
		offset.y = y;
		SDL_BlitSurface(image, &clip, screen, &offset);
	} else {
		SDL_Surface *buttonbg;
		buttonbg = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA,
										width+2*BUTTON_BORDER,
										height+2*BUTTON_BORDER,
										screen->format->BitsPerPixel,
										screen->format->Rmask,
										screen->format->Gmask,
										screen->format->Bmask,
										screen->format->Amask);
		Uint32 bordercolor;
		switch(bstate) {
			case BUTTON_STATE_DEFAULT:
				bordercolor = SDL_MapRGB(buttonbg->format, 255, 255, 255);
				break;
			case BUTTON_STATE_HOVER:
				bordercolor = SDL_MapRGB(buttonbg->format, 0, 100, 25);
				break;
			case BUTTON_STATE_SELECTED:
				bordercolor = SDL_MapRGB(buttonbg->format, 100, 0, 25);
				break;
		}
		SDL_FillRect(buttonbg, NULL, bordercolor);

		SDL_Rect interior;
		interior.x = BUTTON_BORDER;
		interior.y = BUTTON_BORDER;
		interior.w = width;
		interior.h = height;
		if (message == "LE") {
			SDL_FillRect(buttonbg, &interior,
						 SDL_MapRGB(buttonbg->format, 138, 164, 93));

			SDL_Rect clipimg, offset;
			clipimg.x = 0;
			clipimg.y = 0;
			clipimg.w = height;
			clipimg.h = width;
			offset.x = (width+2*BUTTON_BORDER-width)/2;
			offset.y = (height+2*BUTTON_BORDER-height)/2;
			SDL_BlitSurface(image, &clipimg, buttonbg, &offset);
		} else {
			Uint32 bgcolor;
			switch(type){
				case GAME_COLOR_BLACK:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_BLACK.r,
										 COLOR_BLACK.g,
										 COLOR_BLACK.b);
					break;
				case GAME_COLOR_BLUE:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_BLUE.r,
										 COLOR_BLUE.g,
										 COLOR_BLUE.b);
					break;
				case GAME_COLOR_BROWN:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_BROWN.r,
										 COLOR_BROWN.g,
										 COLOR_BROWN.b);
					break;
				case GAME_COLOR_GREEN:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_GREEN.r,
										 COLOR_GREEN.g,
										 COLOR_GREEN.b);
					break;
				case GAME_COLOR_ORANGE:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_ORANGE.r,
										 COLOR_ORANGE.g,
										 COLOR_ORANGE.b);
					break;
				case GAME_COLOR_RED:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_RED.r,
										 COLOR_RED.g,
										 COLOR_RED.b);
					break;
				case GAME_COLOR_VIOLET:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_VIOLET.r,
										 COLOR_VIOLET.g,
										 COLOR_VIOLET.b);
					break;
				case GAME_COLOR_WHITE:
					bgcolor = SDL_MapRGB(buttonbg->format,
										 COLOR_WHITE.r,
										 COLOR_WHITE.g,
										 COLOR_WHITE.b);
					break;
				default:
					// Other types do not affect background color
					break;
			}
			SDL_FillRect(buttonbg, &interior, bgcolor);
		}
		SDL_Rect offset;
		offset.x = x;
		offset.y = y;
		SDL_BlitSurface(buttonbg, NULL, screen, &offset);
		SDL_FreeSurface(buttonbg);
	}
}
// vim: set ts=4 sts=4 sw=4 noet:
