#ifndef INCLUDED_BUTTON_H
#define INCLUDED_BUTTON_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "Utility.hpp"
#include "SoundManager.hpp"

class Button {
public:
	enum ButtonState {
		BUTTON_STATE_DEFAULT,	/**< Default state of the button */
		BUTTON_STATE_HOVER,		/**< When mouse is hovered over button */
		BUTTON_STATE_SELECTED	/**< When mouse is pressed over button */
	};

	/// \brief Enumeration representing the possible Button types
	enum ButtonType {
		GAME_OBJECT_BAT,
		GAME_OBJECT_BEE,		/**< Swarm of Bees */
		GAME_OBJECT_BUTTERFLY,	/**< Butterfly */
		GAME_OBJECT_CENTIPEDE,	/**< Centipede (rolly-polly) */
		GAME_OBJECT_CHARLIE,	/**< Charlie the Chameleon (human-controlled) */
		GAME_OBJECT_DOOR,		/**< A Door */
		GAME_OBJECT_FLOWER,		/**< Immobile flower, for color changing */
		GAME_OBJECT_GOAL,		/**< The goal tile for the level */
		GAME_OBJECT_MILLIPEDE,	/**< Millipede (Killer bug!!) */
		GAME_OBJECT_SNAKE,		/**< Snake */
		GAME_OBJECT_SWITCH,		/**< Switch */
		GAME_OBJECT_TIGERCUB,	/**< Tiger Cub */
		GAME_OBJECT_SCORPION,	/**< Scorpion */
		TILE_BRIDGE,			///< Bridge Tile
		TILE_LAND,				///< Land Tile
		TILE_ROCK,				///< Rock Tile
		TILE_TREE,				///< Tree Tile
		TILE_WATER,				///< Water Tile
		GAME_COLOR_BLACK,
		GAME_COLOR_BLUE,
		GAME_COLOR_BROWN,
		GAME_COLOR_GREEN,
		GAME_COLOR_ORANGE,
		GAME_COLOR_RED,
		GAME_COLOR_VIOLET,
		GAME_COLOR_WHITE,
		NONE
	};

	Button(int x, // (0,0) is top left of button
		   int y,
		   int height,
		   int width,
		   std::string message,
		   std::string path);

	Button(int x, // (0,0) is top left of button
		   int y,
		   int height,
		   int width,
		   std::string message,
		   ButtonType type);

	virtual ~Button();

	virtual void Draw(SDL_Surface *screen) const;
	ButtonState GetState() const;
	void SetState(ButtonState state);
	bool OnButton(const int mx, const int my) const;
	/** \brief Set the tile state to default */
	void Clear();
	ButtonType GetType() const;
	bool GetColor() const {return color;}
	int GetX() const { return x; }
	int GetY() const { return y; }
	int GetHeight() const { return height; }
	int GetWidth() const { return width; }
	
protected:
	int x;
	int y;
	int height;
	int width;
	SDL_Surface *image;
	std::string message;

private:
	ButtonState bstate;
	ButtonType type;
	bool color;

	static Mix_Chunk *hoverNoise;
	static Mix_Chunk *clickNoise;
	static unsigned int buttoncount;
};

inline Button::ButtonState Button::GetState() const { return bstate; }
inline Button::ButtonType Button::GetType() const { return type; }
inline void Button::Clear() { bstate = BUTTON_STATE_DEFAULT; }
inline void Button::SetState(ButtonState state)
{
	if (bstate == state)
		return;

	bstate = state;
	if (bstate == BUTTON_STATE_HOVER &&
		!SoundManager::IsChunkPlaying(clickNoise)) {
		SoundManager::PlayChunk(hoverNoise);
	} else if (bstate == BUTTON_STATE_SELECTED) {
		if (message != "NOSFX")
			SoundManager::PlayChunk(clickNoise);
	}

}

inline bool Button::OnButton(const int mx, const int my) const
{
	return mx >= x && mx <= x+width &&
		   my >= y && my <= y+height;
}

#endif /* INCLUDED_BUTTON_H */
// vim: set ts=4 sts=4 sw=4 noet:
