/// \file Charlie.cpp Source file for the Charlie class

#include <sstream>
#include "Charlie.hpp"

SDL_Surface *Charlie::charlie_black = 0;
SDL_Surface *Charlie::charlie_blue = 0;
SDL_Surface *Charlie::charlie_brown = 0;
SDL_Surface *Charlie::charlie_green = 0;
SDL_Surface *Charlie::charlie_orange = 0;
SDL_Surface *Charlie::charlie_red = 0;
SDL_Surface *Charlie::charlie_violet = 0;
SDL_Surface *Charlie::charlie_white = 0;
SDL_Surface *Charlie::charlie_yellow = 0;

unsigned int Charlie::charliecounter = 0;

Charlie::Charlie(Level *l, Vec2f p, ColorRGBA c) :
	GameObject(l, p, c, CHARLIE_DEFAULT_MAX_SPEED, CHARLIE_DEFAULT_RADIUS),
   	flowerColor(COLOR_GREEN),
	anim_cnt(0),
	anim_timer(0),
	uservel(0.0f,0.0f)
{
	if (charliecounter++ == 0) {
		charlie_black = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-black.png",
								   CREATURE_ROTATIONS,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		charlie_blue = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-blue.png",
								  CREATURE_ROTATIONS,
								  level->GetTileWidth(),
								  level->GetTileHeight());
		charlie_brown = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-brown.png",
								   CREATURE_ROTATIONS,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		charlie_green = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-green.png",
								   CREATURE_ROTATIONS,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		charlie_orange = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-orange.png",
									CREATURE_ROTATIONS,
									level->GetTileWidth(),
									level->GetTileHeight());
		charlie_red = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-red.png",
								 CREATURE_ROTATIONS,
								 level->GetTileWidth(),
								 level->GetTileHeight());
		charlie_violet = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-violet.png",
									CREATURE_ROTATIONS,
									level->GetTileWidth(),
									level->GetTileHeight());
		charlie_white = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-white.png",
								   CREATURE_ROTATIONS,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		charlie_yellow = LoadSprite(PKGDATADIR"Artwork/creatures/charlie/charlie-yellow.png",
									CREATURE_ROTATIONS,
									level->GetTileWidth(),
									level->GetTileHeight());
	}

	image = UpdateImage(GetColor());
}

Charlie::~Charlie()
{
	if (--charliecounter == 0) {
		SDL_FreeSurface(charlie_black);
		SDL_FreeSurface(charlie_blue);
		SDL_FreeSurface(charlie_brown);
		SDL_FreeSurface(charlie_green);
		SDL_FreeSurface(charlie_orange);
		SDL_FreeSurface(charlie_red);
		SDL_FreeSurface(charlie_violet);
		SDL_FreeSurface(charlie_white);
		SDL_FreeSurface(charlie_yellow);
	}
}

void Charlie::Run()
{
	if (disabled)
		return;

	Vec2f input_velocity = Vec2f(0.0f, 0.0f);
	
	if (flowerColor != GetColor()) {
		prevColor = GetColor();
		SetColor(flowerColor);				//color change
		anim_cnt = 255;
		image = UpdateImage(GetColor());
	}

	if (uservel.Length() > 0.5f)
		velocity = uservel * max_speed/uservel.Length();
	else
		velocity = Vec2f(0.0f,0.0f);

	position += velocity;
	Vec2f initial_pos = position;
	//checking collision with static objects
	if (level->GetTile(position)->IsCollidingWith(this)){
		position = initial_pos - Vec2f(velocity.x,0);
		if (level->GetTile(position)->IsCollidingWith(this)){
			position = initial_pos - Vec2f(0,velocity.y);
			if (level->GetTile(position)->IsCollidingWith(this))
				position = initial_pos - velocity;
		}
	}

	if (!(velocity.x == 0 && velocity.y == 0)) {
		image = UpdateImage(GetColor());
		anim_timer = (anim_timer + 1) % CHARLIE_ANIMATION_SPEED;
		if (anim_timer == 0)
			anim_index = (anim_index + 1) % 8;
	}

	if (anim_cnt != 0)
		anim_cnt -= 5;
}

void Charlie::KeyboardEvent(SDL_KeyboardEvent *event)
{
	switch (event->keysym.sym) {
		case SDLK_a:
		case SDLK_LEFT:
			if (event->type == SDL_KEYDOWN)
				uservel.x = -1.0f;
			else if (event->type == SDL_KEYUP)
				uservel.x = 0.0f;
			break;
		case SDLK_s:
		case SDLK_DOWN:
			if (event->type == SDL_KEYDOWN)
				uservel.y = 1.0f;
			else if (event->type == SDL_KEYUP)
				uservel.y = 0.0f;
			break;
		case SDLK_d:
		case SDLK_RIGHT:
			if (event->type == SDL_KEYDOWN)
				uservel.x = 1.0f;
			else if (event->type == SDL_KEYUP)
				uservel.x = 0.0f;
			break;
		case SDLK_w:
		case SDLK_UP:
			if (event->type == SDL_KEYDOWN)
				uservel.y = -1.0f;
			else if (event->type == SDL_KEYUP)
				uservel.y = 0.0f;
			break;
		case SDLK_SPACE:
			if (event->type == SDL_KEYDOWN)
				flowerColor = COLOR_GREEN;
			break;
		default:
			// Other keys do not affect movement
			break;
	}
}

void Charlie::MouseMotionEvent(SDL_MouseMotionEvent *event)
{
}

void Charlie::MouseButtonEvent(SDL_MouseButtonEvent *event)
{
}

void Charlie::Reset()
{
	GameObject::Reset();

	flowerColor = COLOR_GREEN;
	SetColor(COLOR_GREEN);

	image = UpdateImage(GetColor());
	anim_cnt = 0;
	anim_timer = 0;
	anim_index = 0;

	max_speed = CHARLIE_DEFAULT_MAX_SPEED;
	radius = CHARLIE_DEFAULT_RADIUS;
}

bool Charlie::GetInvisible()
{
	if (level->GetTile(Vec2f(position.x,position.y))->HasColor()) {
		ColorRGBA levelcolor = level->GetTile(Vec2f(position.x,position.y))->GetColor();
		return GetColor() == levelcolor;
	} else
		return false;
}

std::string Charlie::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"chameleon\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";
	
	xml << "/>\n";

	return xml.str();
}

SDL_Surface* Charlie::UpdateImage(ColorRGBA c)
{
	if (c == COLOR_BLACK)
		return charlie_black;
	else if (c == COLOR_BLUE)
		return charlie_blue;
	else if (c == COLOR_BROWN)
		return charlie_brown;
	else if (c == COLOR_GREEN)
		return charlie_green;
	else if (c == COLOR_ORANGE)
		return charlie_orange;
	else if (c == COLOR_RED)
		return charlie_red;
	else if (c == COLOR_VIOLET)
		return charlie_violet;
	else if (c == COLOR_WHITE)
		return charlie_white;
	else
		return charlie_yellow;
}
// vim: set ts=4 sts=4 sw=4 noet:
