/// \file DialogBoxes.hpp Header file for the DialogBoxes class

/// Include guard for DialogBoxes.hpp
#ifndef INCLUDED_DIALOGBOXES_H
#define INCLUDED_DIALOGBOXES_H 1

#include <string>

/**
 * \brief Class for interacting with the user through window-system dialog boxes
 */
class DialogBoxes
{
public:
	/** \brief Destroy DialogBoxes */
	virtual ~DialogBoxes() {}

	/** \brief Print an warning message to the user
	 *
	 * \param format Format for the warning message
	 */
	virtual void WarningMessage(const char *format, ...) =0;

	/** \brief Print an error message to the user
	 *
	 * \param format Format for the error message
	 */
	virtual void ErrorMessage(const char *format, ...) =0;

	/** \brief Prompt the user with a Save As dialog box to save a Level
	 *
	 * \param filename Suggested filename, modified to be the filename chosen
	 * \param dirname Directory where dialog should start (defaults to directory
	 *                containing filename, if provided)
	 * \return true if the user selected a file, false if the user canceled
	 */
	virtual bool PromptSaveLevel(std::string& filename,
								 const char *dirname = 0) =0;

	/** \brief Prompt the user with an Open File dialog box to open a Level
	 *
	 * \param filename Suggested filename, modified to be the filename chosen
	 * \param dirname Directory where dialog should start (defaults to directory
	 *                containing filename, if provided)
	 * \return true if the user selected a file, false if the user canceled
	 */
	virtual bool PromptOpenLevel(std::string& filename,
								 const char *dirname = 0) =0;
};

#endif /* INCLUDED_DIALOGBOXES_H */
// vim: set ts=4 sts=4 sw=4 noet:
