/// \file DialogBoxesGTK.cpp Source file for the DialogBoxesGTK class

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cstdio>
#include <gtk/gtk.h>

#include "DialogBoxesGTK.hpp"

DialogBoxesGTK::DialogBoxesGTK(int *argc, char ***argv)
{
	gtk_init(argc, argv);
}

void DialogBoxesGTK::WarningMessage(const char *format, ...)
{
	va_list ap;

	va_start(ap, format);

	bool mustfreemessage = false;
	char *message;
#ifdef HAVE_VASPRINTF
	mustfreemessage = true;
	vasprintf(&message, format, ap);
#elif defined(_MSC_VER)
	mustfreemessage = true;
	message = reinterpret_cast<char *>(malloc(_vscprintf(format, ap) + 1));
	if (message == NULL)
		return;
	vsprintf(message, format, ap);
#else
# error Please define a variadic formatting function
#endif
	va_end(ap);

	GtkWidget *dialog;
	dialog = gtk_message_dialog_new(NULL,
									GTK_DIALOG_DESTROY_WITH_PARENT,
									GTK_MESSAGE_WARNING,
									GTK_BUTTONS_OK,
									message);

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if (mustfreemessage)
		free(message);

	HandlePendingEvents();
}

void DialogBoxesGTK::ErrorMessage(const char *format, ...)
{
	va_list ap;

	va_start(ap, format);

	bool mustfreemessage = false;
	char *message;
#ifdef HAVE_VASPRINTF
	mustfreemessage = true;
	vasprintf(&message, format, ap);
#elif defined(_MSC_VER)
	mustfreemessage = true;
	message = static_cast<char *>(malloc(_vscprintf(format, ap) + 1));
	if (message == NULL)
		return;
	vsprintf(message, format, ap);
#else
# error Please define a variadic formatting function
#endif
	va_end(ap);

	GtkWidget *dialog;
	dialog = gtk_message_dialog_new(NULL,
									GTK_DIALOG_DESTROY_WITH_PARENT,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_OK,
									message);

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if (mustfreemessage)
		free(message);

	HandlePendingEvents();
}

bool DialogBoxesGTK::PromptSaveLevel(std::string& filename,
									 const char *dirname)
{
	GtkWidget *dialog;
	dialog = gtk_file_chooser_dialog_new("Save Level File",
										 NULL,
										 GTK_FILE_CHOOSER_ACTION_SAVE,
										 GTK_STOCK_CANCEL,
										 GTK_RESPONSE_CANCEL,
										 GTK_STOCK_SAVE,
										 GTK_RESPONSE_ACCEPT,
										 NULL);

	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog),
												   TRUE);
	if (dirname)
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), dirname);

	if (filename.length() > 0)
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),
									  filename.c_str());

	bool status;
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		char *cfname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		filename = cfname;
		g_free(cfname);
		status = true;
	} else
		status = false;

	gtk_widget_destroy(dialog);

	HandlePendingEvents();

	return status;
}

bool DialogBoxesGTK::PromptOpenLevel(std::string& filename,
									 const char *dirname)
{
	GtkWidget *dialog;
	dialog = gtk_file_chooser_dialog_new("Save Level File",
										 NULL,
										 GTK_FILE_CHOOSER_ACTION_OPEN,
										 GTK_STOCK_CANCEL,
										 GTK_RESPONSE_CANCEL,
										 GTK_STOCK_OPEN,
										 GTK_RESPONSE_ACCEPT,
										 NULL);

	if (dirname)
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), dirname);

	if (filename.length() > 0)
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),
									  filename.c_str());

	bool status;
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		char *cfname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		filename = cfname;
		g_free(cfname);
		status = true;
	} else
		status = false;

	gtk_widget_hide_all(dialog);
	gtk_widget_destroy(dialog);

	HandlePendingEvents();

	return status;
}

void DialogBoxesGTK::HandlePendingEvents(void)
{
	while (gtk_events_pending())
		gtk_main_iteration();
}

// vim: set ts=4 sts=4 sw=4 noet:
