/// \file FileConfiguration.cpp Source file for the FileConfiguration class

#include <cstdlib>
#include <iostream>
#include <fstream>
#include <stdexcept>

#include "FileConfiguration.hpp"

std::string FileConfiguration::DefaultLocation(void)
{
#ifdef _WIN32
	const char *appdata = getenv("APPDATA");
	if (appdata != NULL)
		return std::string(appdata) + "\\Geuka\\Geuka.ini";

	const char *home = getenv("HOME");
	if (home != NULL)
		return std::string(home) + "\\Application Data\\Geuka\\Geuka.ini";

	return std::string("Geuka.ini");
#else
	const char *home = getenv("HOME");
	if (home != NULL)
		return std::string(home) + "/.geukarc";

	return ".geukarc";
#endif
}

FileConfiguration::FileConfiguration() :
	Configuration(),
	configfile(FileConfiguration::DefaultLocation())
{
}

FileConfiguration::FileConfiguration(std::string configfile) :
	Configuration(),
	configfile(configfile)
{
	LoadFile(configfile);
}

FileConfiguration::~FileConfiguration()
{
}

void FileConfiguration::LoadFile(std::string configfile)
{
	// TODO:  Make this parser more tolerant
	std::ifstream cfile(configfile.c_str());
	if (!cfile) {
		std::string errormsg = "Cannot open ";
		errormsg += configfile;
		throw std::runtime_error(errormsg);
	}

	std::string field;
	char eq;
	while (cfile >> field) {
		cfile >> eq;
		if (eq != '=') {
			std::string errmsg = "Unexpected token \'";
			errmsg + eq + "\' encountered.  \'=\' expected.";
			throw new std::runtime_error(errmsg);
		}

		if (field == "levelscompleted")
			cfile >> levelscompleted;
		else if (field == "fullscreen")
			cfile >> fullscreen;
		else if (field == "windowwidth") {
			forcevideo = true;
			cfile >> windowwidth;
		} else if (field == "windowheight") {
			forcevideo = true;
			cfile >> windowheight;
		} else if (field == "colordepth") {
			forcevideo = true;
			cfile >> colordepth;
		} else if (field == "soundon")
			cfile >> soundon;
		else
			std::cerr << "Unrecognized configuration option \""
				<< field << "\"\n";
	}
}

void FileConfiguration::Save() const
{
	SaveTo(configfile);
}

void FileConfiguration::SaveTo(std::string configfile) const
{
	std::ofstream cfile(configfile.c_str());
	if (!cfile) {
		std::string errormsg = "Cannot open ";
		errormsg += configfile;
		throw std::runtime_error(errormsg);
	}

	cfile << "levelscompleted = " << levelscompleted << '\n';
	cfile << "fullscreen = " << fullscreen << '\n';
	cfile << "soundon = " << soundon << '\n';
	if (forcevideo) {
		cfile << "windowwidth = " << windowwidth << '\n';
		cfile << "windowheight = " << windowheight << '\n';
		cfile << "colordepth = " << colordepth << '\n';
	}
}

// vim: set ts=4 sts=4 sw=4 noet:
