/// \file FileConfiguration.hpp Header file for the file-based Configuration class

/// Include guard for FileConfiguration.hpp
#ifndef INCLUDED_FILECONFIGURATION_H
#define INCLUDED_FILECONFIGURATION_H 1

#include "Configuration.hpp"

/**
 * \brief Class representing the configuration information for Geuka
 */
class FileConfiguration : public Configuration {
public:
	/** \brief Gets the default configuration file location for the
	 * platform on which Geuka is running */
	static std::string DefaultLocation(void);

	/** \brief Construct a FileConfiguration with default values
	 *  (and default configuration file location) */
	FileConfiguration();

	/** \brief Construct a FileConfiguration from saved values
	 *
	 * This function initializes the game configuration from the information
	 * saved in a configuration file.
	 */
	FileConfiguration(std::string configfile);

	/** \brief Destructs this FileConfiguration instance without saving the
	 * configuration to disk.
	 */
	virtual ~FileConfiguration();

	void Save() const;

	void SaveTo(std::string configfile) const;

private:
	std::string configfile;

	void LoadFile(std::string configfile);
};

#endif /* INCLUDED_FILECONFIGURATION_H */
// vim: set ts=4 sts=4 sw=4 noet:
