/// \file Flower.cpp Source file for the Flower class

#include <sstream>

#include "Flower.hpp"

SDL_Surface* Flower::flower_black;
SDL_Surface* Flower::flower_blue;
SDL_Surface* Flower::flower_brown;
SDL_Surface* Flower::flower_green;
SDL_Surface* Flower::flower_orange;
SDL_Surface* Flower::flower_red;
SDL_Surface* Flower::flower_violet;
SDL_Surface* Flower::flower_white;
SDL_Surface* Flower::flower_yellow;

unsigned int Flower::flowercount = 0;

Flower::Flower(Level *l, Vec2f p, ColorRGBA c, bool dayo, bool nighto, int group) :
    GameObject(l, p, c, FLOWER_DEFAULT_MAX_SPEED, FLOWER_DEFAULT_RADIUS),
   	switchgroup(group),
	dayopen(dayo),
	nightopen(nighto),
   	switchedopen(false),
   	wanttoclose(false),
	closing_animation_timer(0),
	closed((dayopen && !level->IsItDay()) || (nightopen && !level->IsItDay()))
{
	if (flowercount++ == 0) {
		flower_black = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-black.png",
								  1,
								  level->GetTileWidth(),
								  level->GetTileHeight());
		flower_blue = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-blue.png",
								 1,
								 level->GetTileWidth(),
								 level->GetTileHeight());
		flower_brown = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-brown.png",
								  1,
								  level->GetTileWidth(),
								  level->GetTileHeight());
		flower_green = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-green.png",
								  1,
								  level->GetTileWidth(),
								  level->GetTileHeight());
		flower_orange = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-orange.png",
								   1,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		flower_red = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-red.png",
								1,
								level->GetTileWidth(),
								level->GetTileHeight());
		flower_violet = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-violet.png",
								   1,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		flower_white = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-white.png",
								  1,
								  level->GetTileWidth(),
								  level->GetTileHeight());
		flower_yellow = LoadSprite(PKGDATADIR"Artwork/objects/flower/flower-yellow.png",
								   1,
								   level->GetTileWidth(),
								   level->GetTileHeight());
	}

	if(c == COLOR_BLACK)
		image = flower_black;
	else if(c == COLOR_BLUE)
		image = flower_blue;
	else if(c == COLOR_BROWN)
		image = flower_brown;
	else if(c == COLOR_GREEN)
		image = flower_green;
	else if(c == COLOR_ORANGE)
		image = flower_orange;
	else if(c == COLOR_RED)
		image = flower_red;
	else if(c == COLOR_VIOLET)
		image = flower_violet;
	else if(c == COLOR_WHITE)
		image = flower_white;
	else
		image = flower_yellow;

	UpdateImage();
}

Flower::~Flower()
{
	if (--flowercount == 0) {
		SDL_FreeSurface(flower_black);
		SDL_FreeSurface(flower_blue);
		SDL_FreeSurface(flower_brown);
		SDL_FreeSurface(flower_green);
		SDL_FreeSurface(flower_orange);
		SDL_FreeSurface(flower_red);
		SDL_FreeSurface(flower_violet);
		SDL_FreeSurface(flower_white);
		SDL_FreeSurface(flower_yellow);
	}
}

void Flower::Run()
{
	if (disabled)
		return;

	if (closing_animation_timer > 0)
	{
		--closing_animation_timer;
		if (closing_animation_timer == 0)
			UpdateImage();
	}

	if (switchedopen)
		return;

	if (wanttoclose) {
		if (CanClose()) {
			if(closed == false)
				closing_animation_timer = FLOWER_ANIMATION_TIME;
			closed = true;
			UpdateImage();
			wanttoclose = false;
		}
	} else if ((closed && level->IsItDay() && dayopen) ||
			   (closed && !level->IsItDay() && nightopen)) {
	    if (closed == true)
			closing_animation_timer = FLOWER_ANIMATION_TIME;
		closed = false;
		UpdateImage();
	} else if ((!closed && level->IsItDay() && !dayopen) ||
			   (!closed && !level->IsItDay() && !nightopen)) {
		if (CanClose()) {
			if(closed == false)
				closing_animation_timer = FLOWER_ANIMATION_TIME;
			closed = true;
			UpdateImage();
		} else {
			wanttoclose = true;
		}
	}

	if(closing_animation_timer >= 0)
	{
		--closing_animation_timer;
		if(closing_animation_timer >= -1)
			UpdateImage();
	}
}


void Flower::Reset()
{
	GameObject::Reset();

	if ((dayopen && level->IsItDay()) ||
		(nightopen && !level->IsItDay())) {
		closed = false;
	} else {
		closed = true;
	}

	wanttoclose = false;
	max_speed = FLOWER_DEFAULT_MAX_SPEED;
	radius = FLOWER_DEFAULT_RADIUS;

	UpdateImage();
}

void Flower::SetState(bool s)
{
	if (wanttoclose && !s) {
		wanttoclose = false;
		return;
	}

	switchedopen = s;

	if (switchedopen ||
		(dayopen && level->IsItDay()) ||
		(nightopen && !level->IsItDay())) {
		if(closed == true)
			closing_animation_timer = FLOWER_ANIMATION_TIME;
		closed = false;
		UpdateImage();
	} else if (!CanClose()) {
		wanttoclose = false;
	} else {
		if(closed == false)
			closing_animation_timer = FLOWER_ANIMATION_TIME;
		closed = true;
		UpdateImage();
	}
}

std::string Flower::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<object x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"flower\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";
	
	if (dayopen) {
		if (nightopen)
			xml << " open=\"both\"";
		else
			xml << " open=\"day\"";
	} else {
		if (nightopen)
			xml << " open=\"night\"";
		else
			xml << " open=\"neither\"";
	}

	if (switchgroup > 0)
		xml << " group=\"" << switchgroup << "\"";

	xml << "/>\n";

	return xml.str();
}

bool Flower::CanClose() const
{
	for (unsigned int x = max(tile_position.x-1,0);
		 x < min(static_cast<unsigned>(tile_position.x+1),level->GetCols()-1);
		 x++) {
		for (unsigned int y = max(tile_position.y-1,0);
			 y < min(static_cast<unsigned>(tile_position.y+1),level->GetRows()-1);
			 y++) {
			Tile *tile = level->GetTile(x, y);
			for (unsigned int i=0; i<tile->objects.size(); i++) {
				GameObject *object = tile->objects[i];
				if (this != object && !object->GetDisabled() &&
					GetOverlap(object) > -0.1f)
					// Note: 0.1f buffer to prevent knockback
					return false;
			}
		}
	}
	return true;
}

void Flower::UpdateImage()
{
	if (closing_animation_timer > 0)
		anim_index = 1;
	else if (closed)
		anim_index = 2;
	else
		anim_index = 0;
}

// vim: set ts=4 sts=4 sw=4 noet:
