/// \file LevelEditor.cpp Source file for the LevelEditor class

#include "SDL_ttf.h"

#include "LevelParser.hpp"
#include "LevelWriter.hpp"
#include "LevelEditor.hpp"

LevelEditor::LevelEditor(const char *const levelfile) :
	Charlie (40,260,32,32,"LE",Button::GAME_OBJECT_CHARLIE),
	Bee (120,260,32,32,"LE",Button::GAME_OBJECT_BEE),
	Snake (200,260,32,32,"LE",Button::GAME_OBJECT_SNAKE),
	TigerCub (280,260,32,32,"LE",Button::GAME_OBJECT_TIGERCUB),
	Centipede (360,260,32,32,"LE",Button::GAME_OBJECT_CENTIPEDE),
	Bat (440,260,32,32,"LE",Button::GAME_OBJECT_BAT),
	Butterfly (40,340,32,32,"LE",Button::GAME_OBJECT_BUTTERFLY),
	Scorpion (120,340,32,32,"LE",Button::GAME_OBJECT_SCORPION),
	Millipede (200,340,32,32,"LE",Button::GAME_OBJECT_MILLIPEDE),
	Rock (40,540,32,32,"LE",Button::TILE_ROCK),
	Water (120,540,32,32,"LE",Button::TILE_WATER),
	Tree (200,540,32,32,"LE",Button::TILE_TREE),
	Bridge (280,540,32,32,"LE",Button::TILE_BRIDGE),
	Land (360,540,32,32,"LE",Button::TILE_LAND),
	Switch (40,740,32,32,"LE",Button::GAME_OBJECT_SWITCH),
	Door (120,740,32,32,"LE",Button::GAME_OBJECT_DOOR),
	Flower (200,740,32,32,"LE",Button::GAME_OBJECT_FLOWER),
	Goal (280,740,32,32,"LE",Button::GAME_OBJECT_GOAL),
	Black (40,940,32,32,"LE-C",Button::GAME_COLOR_BLACK),
	Blue (90,940,32,32,"LE-C",Button::GAME_COLOR_BLUE),
	Brown (140,940,32,32,"LE-C",Button::GAME_COLOR_BROWN),
	Green (190,940,32,32,"LE-C",Button::GAME_COLOR_GREEN),
	Orange (240,940,32,32,"LE-C",Button::GAME_COLOR_ORANGE),
	Red (290,940,32,32,"LE-C",Button::GAME_COLOR_RED),
	Violet (340,940,32,32,"LE-C",Button::GAME_COLOR_VIOLET),
	White (390,940,32,32,"LE-C",Button::GAME_COLOR_WHITE),
	NewLvl (20,1340,120,210,"",PKGDATADIR"Artwork/ui/level_editor/new.png"),
	LoadLvl (230,1340,120,210,"",PKGDATADIR"Artwork/ui/level_editor/load.png"),
	SaveLvl (440,1340,120,210,"",PKGDATADIR"Artwork/ui/level_editor/save.png"),
	MainMenu (1448,25,120,500,"",PKGDATADIR"Artwork/ui/mainmenu/backtomain.png"),
	DayLNext (564,1053,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	DayLPrev (320,1053,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	StartNext (564,1133,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	StartPrev (320,1133,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	TypeNext (564,1213,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	TypePrev (320,1213,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	Name (25,25,120,868,"NOSFX",PKGDATADIR"Artwork/ui/level_editor/name.png"),
	TimeLimitNext (564,1293,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	TimeLimitPrev (320,1293,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	objSelected(NULL),
	estate(EDITOR_STATE_RUN),
	tileCat(TILE_TYPE_CREATURES),
	typeSelected(Button::NONE),
	showColor(false)
{
	level = new Level();
	LevelParser::parseLevelFile(levelfile, level, Vec2i(673,180));
	level->InitializeEditor();

	creatures_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Creatures.png");
	walls_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Walls.png");
	objects_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Objects.png");
	colors_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Colors.png");
	daylength_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/day_length.png");
	levelstart_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/level_start.png");
	leveltype_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/level_type.png");
	day_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/day.png");
	night_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/night.png");
	goal_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/goal.png");
	time_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/time.png");
	collect_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/collect.png");
	s10s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/10s.png");
	s20s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/20s.png");
	s30s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/30s.png");
	s40s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/40s.png");
	s50s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/50s.png");
	s60s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/60s.png");
	timelimit_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/time_limit.png");
	m05min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/05min.png");
	m1min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/1min.png");
	m15min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/15min.png");
	m2min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/2min.png");
	m3min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/3min.png");
	m5min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/5min.png");

	creatures.push_back(&Charlie);
	creatures.push_back(&Bee);
	creatures.push_back(&Snake);
	creatures.push_back(&TigerCub);
	creatures.push_back(&Centipede);
	creatures.push_back(&Bat);
	creatures.push_back(&Butterfly);
	creatures.push_back(&Scorpion);
	creatures.push_back(&Millipede);

	walls.push_back(&Rock);
	walls.push_back(&Water);
	walls.push_back(&Tree);
	walls.push_back(&Bridge);
	walls.push_back(&Land);

	special.push_back(&Switch);
	special.push_back(&Door);
	special.push_back(&Flower);
	special.push_back(&Goal);

	colors.push_back(&Black);
	colors.push_back(&Blue);
	colors.push_back(&Brown);
	colors.push_back(&Green);
	colors.push_back(&Orange);
	colors.push_back(&Red);
	colors.push_back(&Violet);
	colors.push_back(&White);

	menu.push_back(&NewLvl);
	menu.push_back(&LoadLvl);
	menu.push_back(&SaveLvl);
	menu.push_back(&MainMenu);

	arrows.push_back(&DayLNext);
	arrows.push_back(&DayLPrev);
	arrows.push_back(&StartNext);
	arrows.push_back(&StartPrev);
	arrows.push_back(&TypeNext);
	arrows.push_back(&TypePrev);
	arrows.push_back(&TimeLimitNext);
	arrows.push_back(&TimeLimitPrev);

	daylength_v.push_back(s10s_txt);
	daylength_v.push_back(s20s_txt);
	daylength_v.push_back(s30s_txt);
	daylength_v.push_back(s40s_txt);
	daylength_v.push_back(s50s_txt);
	daylength_v.push_back(s60s_txt);
	daylength_idx = 0;

	levelstart_v.push_back(day_txt);
	levelstart_v.push_back(night_txt);
	levelstart_idx = 0;

	leveltype_v.push_back(goal_txt);
	leveltype_v.push_back(time_txt);
	leveltype_v.push_back(collect_txt);
	leveltype_idx = 0;

	timelimit_v.push_back(m05min_txt);
	timelimit_v.push_back(m1min_txt);
	timelimit_v.push_back(m15min_txt);
	timelimit_v.push_back(m2min_txt);
	timelimit_v.push_back(m3min_txt);
	timelimit_v.push_back(m5min_txt);
	timelimit_idx = 0;

	typingName = false;

	LevelName = "";
}

LevelEditor::LevelEditor(const int rows, const int cols) :
	Charlie (40,260,32,32,"LE",Button::GAME_OBJECT_CHARLIE),
	Bee (120,260,32,32,"LE",Button::GAME_OBJECT_BEE),
	Snake (200,260,32,32,"LE",Button::GAME_OBJECT_SNAKE),
	TigerCub (280,260,32,32,"LE",Button::GAME_OBJECT_TIGERCUB),
	Centipede (360,260,32,32,"LE",Button::GAME_OBJECT_CENTIPEDE),
	Bat (440,260,32,32,"LE",Button::GAME_OBJECT_BAT),
	Butterfly (40,340,32,32,"LE",Button::GAME_OBJECT_BUTTERFLY),
	Scorpion (120,340,32,32,"LE",Button::GAME_OBJECT_SCORPION),
	Millipede (200,340,32,32,"LE",Button::GAME_OBJECT_MILLIPEDE),
	Rock (40,540,32,32,"LE",Button::TILE_ROCK),
	Water (120,540,32,32,"LE",Button::TILE_WATER),
	Tree (200,540,32,32,"LE",Button::TILE_TREE),
	Bridge (280,540,32,32,"LE",Button::TILE_BRIDGE),
	Land (360,540,32,32,"LE",Button::TILE_LAND),
	Switch (40,740,32,32,"LE",Button::GAME_OBJECT_SWITCH),
	Door (120,740,32,32,"LE",Button::GAME_OBJECT_DOOR),
	Flower (200,740,32,32,"LE",Button::GAME_OBJECT_FLOWER),
	Goal (280,740,32,32,"LE",Button::GAME_OBJECT_GOAL),
	Black (40,940,32,32,"LE-C",Button::GAME_COLOR_BLACK),
	Blue (90,940,32,32,"LE-C",Button::GAME_COLOR_BLUE),
	Brown (140,940,32,32,"LE-C",Button::GAME_COLOR_BROWN),
	Green (190,940,32,32,"LE-C",Button::GAME_COLOR_GREEN),
	Orange (240,940,32,32,"LE-C",Button::GAME_COLOR_ORANGE),
	Red (290,940,32,32,"LE-C",Button::GAME_COLOR_RED),
	Violet (340,940,32,32,"LE-C",Button::GAME_COLOR_VIOLET),
	White (390,940,32,32,"LE-C",Button::GAME_COLOR_WHITE),
	NewLvl (20,1340,120,210,"",PKGDATADIR"Artwork/ui/level_editor/new.png"),
	LoadLvl (230,1340,120,210,"",PKGDATADIR"Artwork/ui/level_editor/load.png"),
	SaveLvl (440,1340,120,210,"",PKGDATADIR"Artwork/ui/level_editor/save.png"),
	MainMenu (1448,25,120,500,"",PKGDATADIR"Artwork/ui/mainmenu/backtomain.png"),
	DayLNext (564,1053,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	DayLPrev (320,1053,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	StartNext (564,1133,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	StartPrev (320,1133,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	TypeNext (564,1213,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	TypePrev (320,1213,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	TimeLimitNext (564,1293,32,32,"",PKGDATADIR"Artwork/ui/level_editor/right_arrow.png"),
	TimeLimitPrev (320,1293,32,32,"",PKGDATADIR"Artwork/ui/level_editor/left_arrow.png"),
	Name (25,25,120,868,"NOSFX",PKGDATADIR"Artwork/ui/level_editor/name.png"),
	objSelected(NULL),
	estate(EDITOR_STATE_RUN),
	tileCat(TILE_TYPE_CREATURES),
	typeSelected(Button::NONE),
	showColor(false)
{
	level = new Level();
	level->InitializeEditor(rows,cols,Vec2i(673,180),1280,1280);

	creatures_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Creatures.png");
	walls_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Walls.png");
	objects_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Objects.png");
	colors_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/Colors.png");
	daylength_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/day_length.png");
	levelstart_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/level_start.png");
	leveltype_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/level_type.png");
	day_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/day.png");
	night_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/night.png");
	goal_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/goal.png");
	time_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/time.png");
	collect_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/collect.png");
	s10s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/10s.png");
	s20s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/20s.png");
	s30s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/30s.png");
	s40s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/40s.png");
	s50s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/50s.png");
	s60s_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/60s.png");
	timelimit_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/time_limit.png");
	m05min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/05min.png");
	m1min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/1min.png");
	m15min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/15min.png");
	m2min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/2min.png");
	m3min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/3min.png");
	m5min_txt = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/5min.png");

	creatures.push_back(&Charlie);
	creatures.push_back(&Bee);
	creatures.push_back(&Snake);
	creatures.push_back(&TigerCub);
	creatures.push_back(&Centipede);
	creatures.push_back(&Bat);
	creatures.push_back(&Butterfly);
	creatures.push_back(&Scorpion);
	creatures.push_back(&Millipede);

	walls.push_back(&Rock);
	walls.push_back(&Water);
	walls.push_back(&Tree);
	walls.push_back(&Bridge);
	walls.push_back(&Land);

	special.push_back(&Switch);
	special.push_back(&Door);
	special.push_back(&Flower);
	special.push_back(&Goal);

	colors.push_back(&Black);
	colors.push_back(&Blue);
	colors.push_back(&Brown);
	colors.push_back(&Green);
	colors.push_back(&Orange);
	colors.push_back(&Red);
	colors.push_back(&Violet);
	colors.push_back(&White);

	menu.push_back(&NewLvl);
	menu.push_back(&LoadLvl);
	menu.push_back(&SaveLvl);
	menu.push_back(&MainMenu);

	arrows.push_back(&DayLNext);
	arrows.push_back(&DayLPrev);
	arrows.push_back(&StartNext);
	arrows.push_back(&StartPrev);
	arrows.push_back(&TypeNext);
	arrows.push_back(&TypePrev);
	arrows.push_back(&TimeLimitNext);
	arrows.push_back(&TimeLimitPrev);

	daylength_v.push_back(s10s_txt);
	daylength_v.push_back(s20s_txt);
	daylength_v.push_back(s30s_txt);
	daylength_v.push_back(s40s_txt);
	daylength_v.push_back(s50s_txt);
	daylength_v.push_back(s60s_txt);
	daylength_idx = 0;

	levelstart_v.push_back(day_txt);
	levelstart_v.push_back(night_txt);
	levelstart_idx = 0;

	leveltype_v.push_back(goal_txt);
	leveltype_v.push_back(time_txt);
	leveltype_v.push_back(collect_txt);
	leveltype_idx = 0;

	timelimit_v.push_back(m05min_txt);
	timelimit_v.push_back(m1min_txt);
	timelimit_v.push_back(m15min_txt);
	timelimit_v.push_back(m2min_txt);
	timelimit_v.push_back(m3min_txt);
	timelimit_v.push_back(m5min_txt);
	timelimit_idx = 0;

	typingName = false;

	LevelName = "";
}

LevelEditor::~LevelEditor()
{
	delete level;
	SDL_FreeSurface(creatures_txt);
	SDL_FreeSurface(walls_txt);
	SDL_FreeSurface(objects_txt);
	SDL_FreeSurface(colors_txt);
	SDL_FreeSurface(daylength_txt);
	SDL_FreeSurface(levelstart_txt);
	SDL_FreeSurface(leveltype_txt);
	SDL_FreeSurface(day_txt);
	SDL_FreeSurface(night_txt);
	SDL_FreeSurface(goal_txt);
	SDL_FreeSurface(time_txt);
	SDL_FreeSurface(collect_txt);
	SDL_FreeSurface(s10s_txt);
	SDL_FreeSurface(s20s_txt);
	SDL_FreeSurface(s30s_txt);
	SDL_FreeSurface(s40s_txt);
	SDL_FreeSurface(s50s_txt);
	SDL_FreeSurface(s60s_txt);
	SDL_FreeSurface(timelimit_txt);
	SDL_FreeSurface(m05min_txt);
	SDL_FreeSurface(m1min_txt);
	SDL_FreeSurface(m15min_txt);
	SDL_FreeSurface(m2min_txt);
	SDL_FreeSurface(m3min_txt);
	SDL_FreeSurface(m5min_txt);
}

void LevelEditor::SaveLevel(){
	std::string filename;
	if (dialogs->PromptSaveLevel(filename))
		LevelWriter::writeLevelFile(filename.c_str(), level);
}

void LevelEditor::LoadLevel(){
	std::string filename;
	if (dialogs->PromptOpenLevel(filename)) {
		delete level;
		level = new Level();
		LevelParser::parseLevelFile(filename.c_str(), level, Vec2i(673,180));
		level->InitializeEditor();
	}
}

void LevelEditor::Run()
{
}

void LevelEditor::KeyboardEvent(SDL_KeyboardEvent *event)
{
	if (event->type != SDL_KEYDOWN)
		return;

	if (typingName) {
		switch (event->keysym.sym) {
			case SDLK_BACKSPACE:
				if (LevelName.length() > 0)
					LevelName = LevelName.substr(0,LevelName.size()-1);
				break;
			case SDLK_RETURN:
			case SDLK_TAB:
			case SDLK_ESCAPE:
			case SDLK_KP_ENTER:
				typingName = false;
				break;
			default:
				// FIXME:  Support unicode for real...
				char c = event->keysym.unicode & 0x7F;
				if (!isprint(c))
					return;
				LevelName += c;
				level->setLevelName(LevelName);
				break;
		}
		return;
	}

	switch (estate) {
		case EDITOR_BOARD_STATE_SELECT:
			if (event->keysym.sym == SDLK_DELETE) {
					level->RemoveObject(objSelected);
					objSelected = NULL;
					estate = EDITOR_STATE_RUN;
			}
			if (objSelected->IsPatroller() &&
				(event->keysym.sym == SDLK_RSHIFT ||
				 event->keysym.sym == SDLK_LSHIFT))
				estate = EDITOR_BOARD_STATE_WP;
			break;
		case EDITOR_BOARD_STATE_WP:
			if (event->state == SDL_RELEASED &&
				(event->keysym.sym == SDLK_RSHIFT ||
				 event->keysym.sym == SDLK_LSHIFT))
				estate = EDITOR_BOARD_STATE_SELECT;
			break;
	}

	switch (event->keysym.sym) {
		case SDLK_n:
			estate = EDITOR_STATE_NEW;
			break;
		case SDLK_m:
			estate = EDITOR_STATE_EXIT;
			break;
		case SDLK_l:
			LoadLevel();
			break;
		case SDLK_s:
			SaveLevel();
			break;
		default:
			// Not a hotkey
			break;
	}
}

void LevelEditor::MouseMotionEvent(SDL_MouseMotionEvent *event)
{
	int x = event->x;
	int y = event->y;
	int mx1 = max(0, event->x-level->GetOffset().x);
	int my1 = max(0, event->y-level->GetOffset().y);
	int mx = mx1 / level->GetTileDimension().x;
	int my = my1 / level->GetTileDimension().y;

	if (typingName && event->type == SDL_MOUSEBUTTONDOWN) {
		typingName = !typingName;
		return;
	}

	switch (estate) {
		case EDITOR_BOARD_STATE_DRAG:
			if (objSelected != NULL &&
				objSelected->GetPosition() != Vec2f(mx+0.5f,my+0.5f)) {
				objSelected->SetPosition(Vec2f(mx+0.5f,my+0.5f));
				if (objSelected->IsPatroller())
					dynamic_cast<Patroller *>(objSelected)->ResetWaypoints(objSelected->GetPosition());
			}
			break;
	}

	level->GetEdTile(mx,my)->SetTileStateHover();

	for (unsigned int i=0; i<creatures.size(); i++) {
		if (creatures[i]->OnButton(x,y) &&
		    creatures[i]->GetState() != Button::BUTTON_STATE_SELECTED) {
			creatures[i]->SetState(Button::BUTTON_STATE_HOVER);
		} else if (creatures[i]->GetState() != Button::BUTTON_STATE_SELECTED) {
			creatures[i]->SetState(Button::BUTTON_STATE_DEFAULT);
		}
	}

	for (unsigned int i=0; i<walls.size(); i++) {
		if (walls[i]->OnButton(x,y) &&
			walls[i]->GetState() != Button::BUTTON_STATE_SELECTED) {
			walls[i]->SetState(Button::BUTTON_STATE_HOVER);
		} else if (walls[i]->GetState() != (Button::BUTTON_STATE_SELECTED)) {
			walls[i]->SetState(Button::BUTTON_STATE_DEFAULT);
		}
	}

	for (unsigned int i=0; i<special.size(); i++) {
		if (special[i]->OnButton(x,y) &&
			special[i]->GetState() != (Button::BUTTON_STATE_SELECTED)) {
			special[i]->SetState(Button::BUTTON_STATE_HOVER);
		} else if (special[i]->GetState() != (Button::BUTTON_STATE_SELECTED)) {
			special[i]->SetState(Button::BUTTON_STATE_DEFAULT);
		}
	}

	for (unsigned int i=0; i<menu.size(); i++) {
		if (menu[i]->OnButton(x, y)) {
			menu[i]->SetState(Button::BUTTON_STATE_HOVER);
		} else if (menu[i]->GetState() != Button::BUTTON_STATE_SELECTED) {
			menu[i]->SetState(Button::BUTTON_STATE_DEFAULT);
		}
	}

	for (unsigned int i=0; i<arrows.size(); i++) {
		if (arrows[i]->OnButton(x, y))
			arrows[i]->SetState(Button::BUTTON_STATE_HOVER);
		else if (arrows[i]->GetState() != Button::BUTTON_STATE_SELECTED)
			arrows[i]->SetState(Button::BUTTON_STATE_DEFAULT);
	}

	if (Name.OnButton(x, y)) {
		Name.SetState(Button::BUTTON_STATE_HOVER);
	} else {
		if (typingName) {
			Name.SetState(Button::BUTTON_STATE_SELECTED);
		} else {
			Name.SetState(Button::BUTTON_STATE_DEFAULT);
		}
	}
}

void LevelEditor::MouseButtonEvent(SDL_MouseButtonEvent *event)
{
	int x = event->x;
	int y = event->y;
	int mx1 = max(0, event->x-level->GetOffset().x);
	int my1 = max(0, event->y-level->GetOffset().y);
	int mx = mx1 / level->GetTileDimension().x;
	int my = my1 / level->GetTileDimension().y;

	if (mx1>0 && mx1<1280 && my1>0 && my1<1280) {	// in the board area
		switch(estate) {
			case EDITOR_BOARD_STATE_SELECT:
				if (event->button == SDL_BUTTON_RIGHT &&
					event->type == SDL_MOUSEBUTTONDOWN) {
					objSelected = NULL;
					estate = EDITOR_STATE_RUN;
				} else if (event->button == SDL_BUTTON_RIGHT &&
						   event->type == SDL_MOUSEBUTTONDOWN &&
						   !level->GetTile(mx,my)->objects.empty()) {
					objSelected = level->GetTile(mx,my)->objects.front();
					estate = EDITOR_BOARD_STATE_DRAG;
				} else if (objSelected->IsPatroller() &&
						   event->button == SDL_BUTTON_LEFT &&
						   event->type == SDL_MOUSEBUTTONDOWN) {
						dynamic_cast<Patroller *>(objSelected)->ResetWaypoints(objSelected->GetPosition());
						dynamic_cast<Patroller *>(objSelected)->AddWaypoint(Vec2f(mx+0.5f,my+0.5f));
				}
				break;
			case EDITOR_BOARD_STATE_DRAG:
				if (event->button == SDL_BUTTON_LEFT) {
					estate = EDITOR_BOARD_STATE_SELECT;
					level->UpdateObject(objSelected);
				}
				break;
			case EDITOR_BOARD_STATE_WP:
				if (event->button == SDL_BUTTON_LEFT &&
					event->type == SDL_MOUSEBUTTONDOWN)
					dynamic_cast<Patroller *>(objSelected)->AddWaypoint(Vec2f(mx+0.5f,my+0.5f));
				break;
			case EDITOR_STATE_RUN:
				if (event->button == SDL_BUTTON_LEFT &&
					event->type == SDL_MOUSEBUTTONDOWN) {
					if (typeSelected != Button::NONE &&
						tileTypeSelected == TILE_TYPE_WALLS) {
						level->SetTile(typeSelected, Vec2f(mx,my), colorSelected, showColor);
					} else if (!level->GetTile(mx,my)->objects.empty()) {
							objSelected = level->GetTile(mx,my)->objects.front();
							estate = EDITOR_BOARD_STATE_DRAG;
					}

					if (typeSelected != Button::NONE &&
						tileTypeSelected == TILE_TYPE_CREATURES &&
						!(typeSelected == Button::GAME_OBJECT_CHARLIE &&
						  level->GetCharlie())) {
						level->AddCreature(typeSelected,Vec2f(mx+0.5f,my+0.5f),colorSelected);
						typeSelected = Button::NONE;
						showColor = false;
						ClearSelected();
						objSelected = level->GetTile(mx,my)->objects.front();
						estate = EDITOR_BOARD_STATE_SELECT;
					} else if (typeSelected != Button::NONE && tileTypeSelected == TILE_TYPE_SPECIAL) {
						level->AddSpecial(typeSelected,Vec2f(mx+0.5f,my+0.5f),colorSelected);
						typeSelected = Button::NONE;
						showColor = false;
					} 
				} else if (event->type == SDL_MOUSEBUTTONDOWN &&
						   event->button == SDL_BUTTON_RIGHT) {
					typeSelected = Button::NONE;
					showColor = false;
					ClearSelected();
				}
				break;
		}
	} else {	// in the ui area
		if (event->type == SDL_MOUSEBUTTONDOWN) {
			objSelected = NULL;
			estate = EDITOR_STATE_RUN;
		}

		if (showColor || typeSelected == Button::GAME_OBJECT_BUTTERFLY) {
			for (unsigned int i=0; i<colors.size(); i++) {
				if (colors[i]->OnButton(x,y) &&
					event->type == SDL_MOUSEBUTTONDOWN &&
					event->button == SDL_BUTTON_LEFT) {
					for (unsigned int j=0; j<colors.size(); j++) {
						colors[j]->Clear();
					}
					colors[i]->SetState(Button::BUTTON_STATE_SELECTED);
					switch(colors[i]->GetType()){
						case Button::GAME_COLOR_BLACK:
							colorSelected = COLOR_BLACK;
							break;
						case Button::GAME_COLOR_BLUE:
							colorSelected = COLOR_BLUE;
							break;
						case Button::GAME_COLOR_BROWN:
							colorSelected = COLOR_BROWN;
							break;
						case Button::GAME_COLOR_GREEN:
							colorSelected = COLOR_GREEN;
							break;
						case Button::GAME_COLOR_ORANGE:
							colorSelected = COLOR_ORANGE;
							break;
						case Button::GAME_COLOR_RED:
							colorSelected = COLOR_RED;
							break;
						case Button::GAME_COLOR_VIOLET:
							colorSelected = COLOR_VIOLET;
							break;
						case Button::GAME_COLOR_WHITE:
							colorSelected = COLOR_WHITE;
							break;
					}
				} else if (colors[i]->OnButton(x,y) &&
						   colors[i]->GetState() != Button::BUTTON_STATE_SELECTED) {
					colors[i]->SetState(Button::BUTTON_STATE_HOVER);
				} else if(colors[i]->GetState() != Button::BUTTON_STATE_SELECTED) {
					colors[i]->SetState(Button::BUTTON_STATE_DEFAULT);
				}
			}
		}

		if (event->type != SDL_MOUSEBUTTONDOWN ||
			event->button != SDL_BUTTON_LEFT)
			return;

		for (unsigned int i=0; i<creatures.size(); i++) {
			if (creatures[i]->OnButton(x,y)) {
				ClearSelected();
				creatures[i]->SetState(Button::BUTTON_STATE_SELECTED);
				typeSelected = creatures[i]->GetType();
				tileTypeSelected = TILE_TYPE_CREATURES;
				showColor = creatures[i]->GetColor();
			}
		}

		for (unsigned int i=0; i<walls.size(); i++) {
			if (walls[i]->OnButton(x,y)) {
				ClearSelected();
				walls[i]->SetState(Button::BUTTON_STATE_SELECTED);
				typeSelected = walls[i]->GetType();
				tileTypeSelected = TILE_TYPE_WALLS;
				showColor = walls[i]->GetColor();
			}
		}

		for (unsigned int i=0; i<special.size(); i++) {
			if (special[i]->OnButton(x,y)) {
				ClearSelected();
				special[i]->SetState(Button::BUTTON_STATE_SELECTED);
				typeSelected = special[i]->GetType();
				tileTypeSelected = TILE_TYPE_SPECIAL;
				showColor = special[i]->GetColor();
			}
		}

		for (unsigned int i=0; i<menu.size(); i++) {
			if (menu[i]->OnButton(x, y)) {
				menu[i]->SetState(Button::BUTTON_STATE_SELECTED);
				switch (i) {
					case 0:
						estate = EDITOR_STATE_NEW;
						break;
					case 1:
						LoadLevel();
						break;
					case 2:
						SaveLevel();
						break;
					case 3:
						estate = EDITOR_STATE_EXIT;
						break;
				}
			}
		}

		for (unsigned int i=0; i<arrows.size(); i++) {
			if (arrows[i]->OnButton(x, y)) {
				arrows[i]->SetState(Button::BUTTON_STATE_SELECTED);
				switch (i) {
					case 0:
						daylength_idx = (daylength_idx+1)%daylength_v.size();
						level->daylength = (daylength_idx+1)*10*UPDATES_PER_SECOND;
						if (level->initialdayremaining < 0)
							level->initialdayremaining = -level->daylength;
						else
							level->initialdayremaining = level->daylength;
						break;
					case 1:
						daylength_idx = (daylength_idx+daylength_v.size()-1)%daylength_v.size();
						level->daylength = (daylength_idx+1)*10*UPDATES_PER_SECOND;
						if (level->initialdayremaining < 0)
							level->initialdayremaining = -level->daylength;
						else
							level->initialdayremaining = level->daylength;
						break;
					case 2:
						levelstart_idx = (levelstart_idx+1)%levelstart_v.size();
						if ((level->initialdayremaining < 0 && levelstart_idx == 0) ||
								(level->initialdayremaining > 0 && levelstart_idx == 1))
							level->initialdayremaining *= -1;
						break;
					case 3:
						levelstart_idx = (levelstart_idx+levelstart_v.size()-1)%levelstart_v.size();
						if ((level->initialdayremaining < 0 && levelstart_idx == 0) ||
								(level->initialdayremaining > 0 && levelstart_idx == 1))
							level->initialdayremaining *= -1;
					case 4:
						leveltype_idx = (leveltype_idx+1)%leveltype_v.size();
						level->victorytype = Level::VictoryType(leveltype_idx);
						break;
					case 5:
						leveltype_idx = (leveltype_idx+leveltype_v.size()-1)%leveltype_v.size();
						level->victorytype = Level::VictoryType(leveltype_idx);
						break;
					case 6:
						timelimit_idx = (timelimit_idx+1)%6;
						switch (timelimit_idx) {
							case 0: level->survivaltime = UPDATES_PER_SECOND/2; break;
							case 1: level->survivaltime = UPDATES_PER_SECOND; break;
							case 2: level->survivaltime = 3*UPDATES_PER_SECOND/2; break;
							case 3: level->survivaltime = 2*UPDATES_PER_SECOND; break;
							case 4: level->survivaltime = 3*UPDATES_PER_SECOND; break;
							case 5: level->survivaltime = 5*UPDATES_PER_SECOND; break;
							default: assert(0);
						}
					case 7:
						timelimit_idx = (timelimit_idx+timelimit_v.size()-1)%timelimit_v.size();
						switch (timelimit_idx) {
							case 0: level->survivaltime = UPDATES_PER_SECOND/2; break;
							case 1: level->survivaltime = UPDATES_PER_SECOND; break;
							case 2: level->survivaltime = 3*UPDATES_PER_SECOND/2; break;
							case 3: level->survivaltime = 2*UPDATES_PER_SECOND; break;
							case 4: level->survivaltime = 3*UPDATES_PER_SECOND; break;
							case 5: level->survivaltime = 5*UPDATES_PER_SECOND; break;
							default: assert(0);
						}
				}
			}
		}
	}
}

void LevelEditor::Draw(SDL_Surface *screen)
{
	level->DrawEditor(screen, objSelected);

	if (showColor) {
		for(unsigned int i=0; i<colors.size(); i++)
			colors[i]->Draw(screen);
	}

	if (typeSelected == Button::GAME_OBJECT_BUTTERFLY) {
		Violet.Draw(screen);
		Orange.Draw(screen);
	}

	SDL_Rect offset;
	offset.x = 20;
	offset.y = 180;
	SDL_BlitSurface(creatures_txt, NULL, screen, &offset);
	for (unsigned int i=0; i<creatures.size(); i++)
		creatures[i]->Draw(screen);

	offset.y = 460;
	SDL_BlitSurface(walls_txt, NULL, screen, &offset);
	for (unsigned int i=0; i<walls.size(); i++)
		walls[i]->Draw(screen);

	offset.y = 660;
	SDL_BlitSurface(objects_txt, NULL, screen, &offset);
	for (unsigned int i=0; i<special.size(); i++)
		special[i]->Draw(screen);

	offset.x = 20;
	offset.y = 860;
	SDL_BlitSurface(colors_txt, NULL, screen, &offset);

	for (unsigned int i=0; i<menu.size(); i++)
		menu[i]->Draw(screen);

	for (unsigned int i=0; i<arrows.size(); i++) {
		if (i < 6 || (leveltype_idx == 1)) {
			arrows[i]->Draw(screen);
		}
	}

	Name.Draw(screen);

	// FIXME:  Speed/Quality improvements here
	SDL_Color textcolor;
	textcolor.r = 25;
	textcolor.g = 200;
	textcolor.b = 50;
	SDL_Surface *lname;
	lname = TTF_RenderText_Solid(gamefont, LevelName.c_str(), textcolor);
	offset.x = 250;
	offset.y = 60;
	SDL_BlitSurface(lname, NULL, screen, &offset);
	SDL_FreeSurface(lname);

	offset.x = 20;
	offset.y = 1050;
	SDL_BlitSurface(daylength_txt, NULL, screen, &offset);
	offset.y = 1130;
	SDL_BlitSurface(levelstart_txt, NULL, screen, &offset);
	offset.y = 1210;
	SDL_BlitSurface(leveltype_txt, NULL, screen, &offset);
	if (leveltype_idx == 1) {
		offset.y = 1290;
		SDL_BlitSurface(timelimit_txt, NULL, screen, &offset);
	}

	offset.x = 384;
	offset.y = 1050;
	SDL_BlitSurface(daylength_v[daylength_idx], NULL, screen, &offset);
	offset.y = 1130;
	SDL_BlitSurface(daylength_v[levelstart_idx], NULL, screen, &offset);
	offset.y = 1210;
	SDL_BlitSurface(daylength_v[leveltype_idx], NULL, screen, &offset);
	if (leveltype_idx == 1) {
		offset.y = 1290;
		SDL_BlitSurface(timelimit_v[timelimit_idx], NULL, screen, &offset);
	}
}

void LevelEditor::ClearSelected()
{
	for (unsigned int i=0; i<creatures.size(); i++) {
		creatures[i]->Clear();
	}
	for (unsigned int i=0; i<walls.size(); i++) {
		walls[i]->Clear();
	}
	for (unsigned int i=0; i<special.size(); i++) {
		special[i]->Clear();
	}
}

// vim: set ts=4 sts=4 sw=4 noet:
