/// \file LevelWriter.hpp Header file for the LevelWriter class

/// Include guard for LevelWriter.hpp
#ifndef INCLUDED_LEVELWRITER_H
#define INCLUDED_LEVELWRITER_H 1

#include "Utility.hpp"

class Level;

/**
 * \brief Class used to parse an XML level file and initialize a Level
 */
class LevelWriter
{
public:
	/** \brief Construct and initialize a LevelWriter
	 * \param level The Level to be initialized from the level data
	 */
	LevelWriter(Level *level){}
	/** \brief Destroy a LevelWriter */
	//~LevelWriter();

	/** \brief Write a level file from a given Level
	 *
	 * \param levelfile The name of the level file to write
	 * \param level The Level to be written
	 * \return 0 for success, <0 on error
	 */
	static int writeLevelFile(const char *levelfile, Level *level);

	static std::string waypointsToXML(const std::string& indent,
						const Vec2f position,
						std::vector<Vec2f>::const_iterator start,
						std::vector<Vec2f>::const_iterator end,
						bool duringday = true, bool duringnight = true);
private:
	/** \brief Write the tiles portion of the level file
	 * \param level Level from which the tiles portion should be generated
	 */
	static void writeTiles(Level *level);

	/** \brief Write the creatures portion of the level file
	 * \param level Level from which the creatures portion should be generated
	 */
	static void writeCreatures(Level *level);

	/** \brief Write the objects portion of the level file
	 * \param level Level from which the objects portion should be generated
	 */
	static void writeObjects(Level *level);

	static std::ofstream saved;		///< The output stream to which the saved
									// portions should be written
};

#include "Level.hpp"
#include "Level.hpp"

#endif /* INCLUDED_LEVELWRITER_H */
// vim: set ts=4 sts=4 sw=4 noet:
