/// \file Patroller.hpp Header file for the Patroller interface

/// Include guard for Patroller.hpp
#ifndef INCLUDED_PATROLLER_H
#define INCLUDED_PATROLLER_H 1

/** \brief Interface representing a creature that patrolls the map */
class Patroller {
public:
	/** \brief Add a waypoint to the patroller's list
	 *
	 * This function adds a point to the path that the patroller follows
	 * around the map.
	 * \param waypoint The point to be added
	 * \param duringday Should this point be visited during the day?
	 * \param duringnight Should this point be visited at night?
	 */
	virtual void AddWaypoint(const Vec2f waypoint,
							 bool duringday = true,
							 bool duringnight = true) =0;

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	virtual void DrawWaypoints(SDL_Surface *screen, ColorRGBA color) =0;

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	virtual void ResetWaypoints(Vec2f pos)=0;
};

#endif /* INCLUDED_PATROLLER_H */
// vim: set ts=4 sts=4 sw=4 noet:
