/// \file Scorpion.cpp Source file for the Scorpion class

#include <sstream>

#include "SDL_gfxPrimitives.h"

#include "Scorpion.hpp"
#include "LevelWriter.hpp"

SDL_Surface *Scorpion::scorpion;
SDL_Surface *Scorpion::scorpionSleep;
unsigned int Scorpion::scorpioncount = 0;

Scorpion::Scorpion(Level *l, Vec2f p) :
	GameObject(l, p, SCORPION_DEFAULT_MAX_SPEED, SCORPION_DEFAULT_RADIUS),
   	waypoints(1, p),
	anim_timer(0),
	anchorpoint(p),
	wp_index(0),
	state(SCORPION_STATE_AWAKE)
{
	if (scorpioncount++ == 0) {
		scorpion = LoadSprite(PKGDATADIR"Artwork/creatures/scorpion/scorpion.png",
							  CREATURE_ROTATIONS,
							  level->GetTileWidth(),
							  level->GetTileHeight());
		scorpionSleep = LoadSprite(PKGDATADIR"Artwork/creatures/scorpion/scorpion_asleep.png",
								   CREATURE_ROTATIONS,
								   level->GetTileWidth(),
								   level->GetTileHeight());
	}

	SetOrientation(0.0f);
	image = scorpion;
}

Scorpion::~Scorpion()
{
	if (--scorpioncount == 0) {
		SDL_FreeSurface(scorpion);
		SDL_FreeSurface(scorpionSleep);
	}
}

void Scorpion::Run()
{
	if (disabled)
	   	return;

	Charlie *c = level->GetCharlie();

	//Decide if my state/wp_index has changed
	switch(state) {
		case SCORPION_STATE_AGGRO:
			if((c->GetPosition() - position).Length() < SCORPION_ATTACK_RADIUS &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()))
			{
				state = SCORPION_STATE_ATTACK;
				max_speed = SCORPION_ATTACK_SPEED;
				anchorpoint = position;
			}
			else if((anchorpoint - position).Length() > SCORPION_NIGHT_AGGRO_RADIUS || c->GetInvisible())
			{
				state = SCORPION_STATE_DEAGGRO;
				max_speed = SCORPION_DEFAULT_MAX_SPEED;
			}
			break;
		case SCORPION_STATE_ASLEEP:
			if((c->GetPosition() - position).Length() < SCORPION_ATTACK_RADIUS &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()))
			{
				state = SCORPION_STATE_ATTACK;
				max_speed = SCORPION_ATTACK_SPEED;
				anchorpoint = position;
			}else if((c->GetPosition() - position).Length() < SCORPION_DAY_AGGRO_RADIUS &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()))
			{
				state = SCORPION_STATE_AGGRO;
				max_speed = SCORPION_AGGRO_MAX_SPEED;
				anchorpoint = position;
			}
			else if(!level->IsItDay())
			{
				state = SCORPION_STATE_AWAKE;
				wp_index = 0;
			}
			break;
		case SCORPION_STATE_AWAKE:
			if((c->GetPosition() - position).Length() < SCORPION_ATTACK_RADIUS &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()))
			{
				state = SCORPION_STATE_ATTACK;
				max_speed = SCORPION_ATTACK_SPEED;
				anchorpoint = position;
			}
			else if((c->GetPosition() - position).Length() < SCORPION_NIGHT_AGGRO_RADIUS &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()) )
			{
				state = SCORPION_STATE_AGGRO;
				max_speed = SCORPION_AGGRO_MAX_SPEED;
				anchorpoint = position;
			}
			else if(level->IsItDay()) {
				state = SCORPION_STATE_ASLEEP;
				anim_index = 0;
			}
			else if(waypoints.at(wp_index).x - max_speed <= position.x && position.x <= waypoints.at(wp_index).x + max_speed
				 && waypoints.at(wp_index).y - max_speed <= position.y && position.y <= waypoints.at(wp_index).y + max_speed)
				wp_index = (wp_index + 1)%waypoints.size();
			break;
		case SCORPION_STATE_DEAGGRO:
			if (!c->GetInvisible() &&
				(c->GetPosition() - position).Length() < SCORPION_ATTACK_RADIUS &&
				level->HasLineOfSight(position, c->GetPosition()))
			{
				state = SCORPION_STATE_ATTACK;
				max_speed = SCORPION_ATTACK_SPEED;
				anchorpoint = position;
			}
			else if(!c->GetInvisible() &&
					((level->IsItDay() && (c->GetPosition() - anchorpoint).Length() < SCORPION_DAY_AGGRO_RADIUS) ||
					 (!level->IsItDay() && (c->GetPosition() - anchorpoint).Length() < SCORPION_NIGHT_AGGRO_RADIUS)) &&
					level->HasLineOfSight(position, c->GetPosition()))
			{
				state = SCORPION_STATE_AGGRO;
				max_speed = SCORPION_AGGRO_MAX_SPEED;
			}
			else if(anchorpoint.x - max_speed - 1 <= position.x && position.x <= anchorpoint.x + max_speed + 1
			&& anchorpoint.y - max_speed - 1 <= position.y && position.y <= anchorpoint.y + max_speed + 1)
				if(level->IsItDay())
				{
					state = SCORPION_STATE_AWAKE;
					wp_index = 0;
				}
				else {
					state = SCORPION_STATE_ASLEEP;
					anim_index = 0;
				}
			break;
		case SCORPION_STATE_ATTACK:
			if (c->GetInvisible()) {
				state = SCORPION_STATE_DEAGGRO;
				max_speed = SCORPION_DEFAULT_MAX_SPEED;
			} else if((c->GetPosition() - position).Length() > SCORPION_ATTACK_RADIUS) {
				state = SCORPION_STATE_AGGRO;
				max_speed = SCORPION_AGGRO_MAX_SPEED;
			}
			break;
		default:
			assert(0);
	}

	Vec2f goal;

	//Set goal based on my state
	switch(state) {
		case SCORPION_STATE_AGGRO: goal = c->GetPosition(); break;
		case SCORPION_STATE_ATTACK: goal = c->GetPosition(); break;
		case SCORPION_STATE_ASLEEP: goal = start_position; break;
		case SCORPION_STATE_AWAKE: goal = waypoints.at(wp_index); break;
		case SCORPION_STATE_DEAGGRO: goal = anchorpoint; break;
		default: assert(0);
	}

	//Move

	velocity = level->GetShortestPath(GetPosition(), goal, max_speed);
	position += velocity;

	UpdateImage();
}

void Scorpion::Draw(SDL_Surface *screen)
{
	if (disabled)
		return;

	// Set translucency appropriately
	Uint8 alpha = 0;
	if (level->GetCharlie() != NULL) {
		float distance = (GetPosition() - level->GetCharlie()->GetPosition()).Length();
		if (distance < SCORPION_ALPHA_DIST) {
			alpha = SCORPION_BASE_ALPHA +
						  (255-SCORPION_BASE_ALPHA)*(1-distance/SCORPION_ALPHA_DIST);
		} else {
			alpha = SCORPION_BASE_ALPHA;
		}
	}

	float theta;
	if (velocity.x == 0.0f) {
		if (velocity.y == 0.0f) {
			if (orientation >= 0.0f)
				theta = orientation;
			else
				theta = prevang;
		} else {
			theta = (velocity.y > 0) ? M_PIf/2 : M_PIf*1.5f;
		}
	} else if (velocity.x < 0.0f) {
		theta = atanf(velocity.y/velocity.x) + M_PIf;
	} else {
		theta = atanf(velocity.y/velocity.x);
	}

	if (theta < 0.0f)
		theta += 2*M_PIf;

	// Save angle for later
	prevang = theta;

	unsigned int vindex = static_cast<unsigned int>(theta*4/M_PIf+0.5f)%8;

	SDL_Rect clip, offset;
	clip.x = anim_index*imagesize.x;
	clip.y = vindex*imagesize.y;
	clip.w = imagesize.x;
	clip.h = imagesize.y;
	Vec2i tiledim = level->GetTileDimension();
	Vec2i leveloffset = level->GetOffset();
	offset.x = position.x*tiledim.x+leveloffset.x-imagesize.x/2;
	offset.y = position.y*tiledim.y+leveloffset.y-imagesize.y/2;

	BlitSurfaceWithAlpha(image, &clip, screen, &offset, alpha);
}

void Scorpion::Reset()
{
	this->GameObject::Reset();

	image = scorpion;

	max_speed = SCORPION_DEFAULT_MAX_SPEED;
	radius = SCORPION_DEFAULT_RADIUS;

	wp_index = 0;
	anchorpoint = start_position;
	state = SCORPION_STATE_AWAKE;
	anim_index = 0;
	anim_timer = 0;
}

std::string Scorpion::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"scorpion\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";
	
	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														waypoints.begin(),
														waypoints.end());
	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</creature>\n";

	return xml.str();
}

void Scorpion::UpdateImage()
{
	if(state == SCORPION_STATE_ASLEEP && velocity.x == 0 && velocity.y == 0)
	{
		anim_timer = (anim_timer + 1) % (SCORPION_ANIMATION_SPEED * 2);

		image = scorpionSleep;
		if (anim_timer == 0)
			anim_index = (anim_index + 1) % 3;

		return;
	}

	anim_timer = (anim_timer + 1) % SCORPION_ANIMATION_SPEED;
	if (anim_timer == 0)
		anim_index = (anim_index + 1) % 4;

	image = scorpion;
}

bool Scorpion::IsPatroller(){
	return true;
}

void Scorpion::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = waypoints.begin();
		 i != waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void Scorpion::ResetWaypoints(Vec2f pos){
	waypoints.clear();
	waypoints.push_back(pos);
}
// vim: set ts=4 sts=4 sw=4 noet:
