/// \file Scorpion.hpp Header file for the Scorpion class

/// Include guard for Scorpion.hpp
#ifndef INCLUDED_SCORPION_H
#define INCLUDED_SCORPION_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "GameObject.hpp"
#include "Patroller.hpp"

/**
 * \brief Class representing a scorpion cub
 */
class Scorpion : public GameObject, public Patroller {
public:
	/** \brief Constructs a Scorpion in Level l at position p.
	* \param l a pointer to the Level this Scorpion exists in
	* \param p the Vec2f start_position of this Scorpion
	*/
	Scorpion(Level *l, Vec2f p);

	/** \brief Destructs a Scorpion */
	~Scorpion();

	/** \brief Runs one cycle of this Scorpion.
	*/
	void Run(void);

	/** \brief Resets this Scorpion to its original state in its Level.
	*/
	void Reset(void);

	void Draw(SDL_Surface *screen);

	inline void AddWaypoint(const Vec2f waypoint,
							bool duringday = true, bool duringnight = true) 
	{ waypoints.push_back(waypoint); }

	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the GameObjectType of this Scorpion.
	 * \return the GameObjectType of this Scorpion
	*/
	inline GameObjectType GetType(void) const { return GAME_OBJECT_SCORPION; };

	void PlayCollisionSound() const;

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	bool IsPatroller();

private:
	/** \brief Updates the image of this Scorpion.
	*/
	void UpdateImage(void);

	static SDL_Surface *scorpion;	///< Front-view image of this Scorpion
	static SDL_Surface *scorpionSleep;	///< sleeping image of this scorpion
	static unsigned int scorpioncount;	///< Number of Scorpions created

	std::vector<Vec2f> waypoints;	///< The waypoints of this Scorpion
	int wp_index;					///< The current index of this Scorpion's waypoints
	Vec2f anchorpoint;				///< The anchorpoint of the Tiger; set when it aggros, returns here when it deaggros.
	/// \brief Describes the mental state of a Scorpion
	enum ScorpionState {
		SCORPION_STATE_AGGRO,
		SCORPION_STATE_ATTACK,
		SCORPION_STATE_ASLEEP,
		SCORPION_STATE_AWAKE,
		SCORPION_STATE_DEAGGRO
	};

	ScorpionState state;			///< The mental state of this Scorpion
	unsigned int anim_timer;		///< How long the Tiger has to pause before moving to the next animation frame
};

inline void Scorpion::PlayCollisionSound() const
{
	// SoundManager::PlayChunk(scorpionRoar);
}

#endif /* INCLUDED_SCORPION_H */
// vim: set ts=4 sts=4 sw=4 noet:
