/// \file Snake.cpp Source file for the Snake class

#include <sstream>

#include "SDL_gfxPrimitives.h"

#include "Snake.hpp"
#include "LevelWriter.hpp"

SDL_Surface *Snake::snake_black;
SDL_Surface *Snake::snake_blue;
SDL_Surface *Snake::snake_brown;
SDL_Surface *Snake::snake_green;
SDL_Surface *Snake::snake_orange;
SDL_Surface *Snake::snake_red;
SDL_Surface *Snake::snake_violet;
SDL_Surface *Snake::snake_white;
SDL_Surface *Snake::snake_yellow;

SDL_Surface *Snake::snake_moving_black;
SDL_Surface *Snake::snake_moving_blue;
SDL_Surface *Snake::snake_moving_brown;
SDL_Surface *Snake::snake_moving_green;
SDL_Surface *Snake::snake_moving_orange;
SDL_Surface *Snake::snake_moving_red;
SDL_Surface *Snake::snake_moving_violet;
SDL_Surface *Snake::snake_moving_white;
SDL_Surface *Snake::snake_moving_yellow;

Mix_Chunk *Snake::snakeBite;
unsigned int Snake::snakecount = 0;

Snake::Snake(Level *l, Vec2f p, ColorRGBA c) : 
	GameObject(l, p, c, SNAKE_DEFAULT_MAX_SPEED, SNAKE_DEFAULT_RADIUS),
	day_waypoints(1,p),
	night_waypoints(1,p),
	day_wp_index(0),
	night_wp_index(0),
	state(SNAKE_STATE_DAY),
	anchorpoint(p),
	anim_timer(0)
{
	// FIXME:  This is inelegant...
	imagesize *= 2;

	if (snakecount++ == 0) {
		snake_blue = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-blue.png",
								CREATURE_ROTATIONS,
								level->GetTileWidth()*2,
								level->GetTileHeight()*2);
		snake_green = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-green.png",
								 CREATURE_ROTATIONS,
								 level->GetTileWidth()*2,
								 level->GetTileHeight()*2);
		snake_red = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-red.png",
							   CREATURE_ROTATIONS,
							   level->GetTileWidth()*2,
							   level->GetTileHeight()*2);
		snake_violet = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-violet.png",
								  CREATURE_ROTATIONS,
								  level->GetTileWidth()*2,
								  level->GetTileHeight()*2);
		snake_black = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-black.png",
								 CREATURE_ROTATIONS,
								 level->GetTileWidth()*2,
								 level->GetTileHeight()*2);
		snake_brown = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-brown.png",
								 CREATURE_ROTATIONS,
								 level->GetTileWidth()*2,
								 level->GetTileHeight()*2);
		snake_orange = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-orange.png",
								  CREATURE_ROTATIONS,
								  level->GetTileWidth()*2,
								  level->GetTileHeight()*2);
		snake_white = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-white.png",
								 CREATURE_ROTATIONS,
								 level->GetTileWidth()*2,
								 level->GetTileHeight()*2);
		snake_yellow = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake_asleep-yellow.png",
								  CREATURE_ROTATIONS,
								  level->GetTileWidth()*2,
								  level->GetTileHeight()*2);

		snake_moving_blue = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-blue.png",
									   CREATURE_ROTATIONS,
									   level->GetTileWidth()*2,
									   level->GetTileHeight()*2);
		snake_moving_green = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-green.png",
										CREATURE_ROTATIONS,
										level->GetTileWidth()*2,
										level->GetTileHeight()*2);
		snake_moving_red = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-red.png",
									  CREATURE_ROTATIONS,
									  level->GetTileWidth()*2,
									  level->GetTileHeight()*2);
		snake_moving_violet = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-violet.png",
										 CREATURE_ROTATIONS,
										 level->GetTileWidth()*2,
										 level->GetTileHeight()*2);
		snake_moving_black = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-black.png",
										CREATURE_ROTATIONS,
										level->GetTileWidth()*2,
										level->GetTileHeight()*2);
		snake_moving_brown = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-brown.png",
										CREATURE_ROTATIONS,
										level->GetTileWidth()*2,
										level->GetTileHeight()*2);
		snake_moving_orange = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-orange.png",
										 CREATURE_ROTATIONS,
										 level->GetTileWidth()*2,
										 level->GetTileHeight()*2);
		snake_moving_white = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-white.png",
										CREATURE_ROTATIONS,
										level->GetTileWidth()*2,
										level->GetTileHeight()*2);
		snake_moving_yellow = LoadSprite(PKGDATADIR"Artwork/creatures/snake/snake-yellow.png",
										 CREATURE_ROTATIONS,
										 level->GetTileWidth()*2,
										 level->GetTileHeight()*2);

		snakeBite = LoadSound(PKGDATADIR"Sounds/SnakeHiss.ogg");
	}

	if (c == COLOR_BLACK) {
		still_image = snake_black;
		moving_image = snake_moving_black;
	} else if (c == COLOR_BLUE) {
		still_image = snake_blue;
		moving_image = snake_moving_blue;
	} else if (c == COLOR_BROWN) {
		still_image = snake_brown;
		moving_image = snake_moving_brown;
	} else if (c == COLOR_GREEN) {
		still_image = snake_green;
		moving_image = snake_moving_green;
	} else if (c == COLOR_ORANGE) {
		still_image = snake_orange;
		moving_image = snake_moving_orange;
	} else if (c == COLOR_RED) {
		still_image = snake_red;
		moving_image = snake_moving_red;
	} else if (c == COLOR_VIOLET) {
		still_image = snake_violet;
		moving_image = snake_moving_violet;
	} else if (c == COLOR_WHITE) {
		still_image = snake_white;
		moving_image = snake_moving_white;
	} else if (c == COLOR_YELLOW) {
		still_image = snake_yellow;
		moving_image = snake_moving_yellow;
	} else {
		assert(0);	// Unrecognized color (does not match colors we have images for)
		still_image = snake_black;
		moving_image = snake_moving_black;
	}

	SetOrientation(0.0f);
	UpdateImage();
}

Snake::~Snake()
{
	if (--snakecount == 0) {
		SDL_FreeSurface(snake_black);
		SDL_FreeSurface(snake_blue);
		SDL_FreeSurface(snake_brown);
		SDL_FreeSurface(snake_green);
		SDL_FreeSurface(snake_orange);
		SDL_FreeSurface(snake_red);
		SDL_FreeSurface(snake_violet);
		SDL_FreeSurface(snake_white);
		SDL_FreeSurface(snake_yellow);
		SDL_FreeSurface(snake_moving_black);
		SDL_FreeSurface(snake_moving_blue);
		SDL_FreeSurface(snake_moving_brown);
		SDL_FreeSurface(snake_moving_green);
		SDL_FreeSurface(snake_moving_orange);
		SDL_FreeSurface(snake_moving_red);
		SDL_FreeSurface(snake_moving_violet);
		SDL_FreeSurface(snake_moving_white);
		SDL_FreeSurface(snake_moving_yellow);
		Mix_FreeChunk(snakeBite);
	}
}

void Snake::Run()
{
	if(disabled) return;

	Charlie *c = level->GetCharlie();

	//Decide if my state/wp_index has changed
	switch(state) {
		case SNAKE_STATE_AGGRO:
			if ((anchorpoint - position).Length() > SNAKE_AGGRO_RADIUS ||
				c->GetColor() == GetColor())
			{
				state = SNAKE_STATE_DEAGGRO;
				max_speed = SNAKE_DEFAULT_MAX_SPEED;
			}
			break;
		case SNAKE_STATE_NIGHT:
			if ((c->GetPosition() - position).Length() < SNAKE_AGGRO_RADIUS &&
				c->GetColor() != GetColor() &&
				level->HasLineOfSight(position, c->GetPosition()) )
			{
				state = SNAKE_STATE_AGGRO;
				max_speed = SNAKE_AGGRO_MAX_SPEED;
				anchorpoint = position;
			}
			else if(level->IsItDay())
			{
				state = SNAKE_STATE_DAY;
				day_wp_index = 0;
			}
			else if(night_waypoints.at(night_wp_index).x - max_speed <= position.x && position.x <= night_waypoints.at(night_wp_index).x + max_speed
				 && night_waypoints.at(night_wp_index).y - max_speed <= position.y && position.y <= night_waypoints.at(night_wp_index).y + max_speed)
				night_wp_index = (night_wp_index + 1)%night_waypoints.size();
			break;
		case SNAKE_STATE_DAY:
			if ((c->GetPosition() - position).Length() < SNAKE_AGGRO_RADIUS &&
				c->GetColor() != GetColor() &&
				level->HasLineOfSight(position, c->GetPosition()) )
			{
				state = SNAKE_STATE_AGGRO;
				max_speed = SNAKE_AGGRO_MAX_SPEED;
				anchorpoint = position;
			}
			else if(!level->IsItDay())
			{
				state = SNAKE_STATE_NIGHT;
				night_wp_index = 0;
			}
			else if(day_waypoints.at(day_wp_index).x - max_speed <= position.x && position.x <= day_waypoints.at(day_wp_index).x + max_speed
				 && day_waypoints.at(day_wp_index).y - max_speed <= position.y && position.y <= day_waypoints.at(day_wp_index).y + max_speed)
				day_wp_index = (day_wp_index + 1)%day_waypoints.size();
			break;
		case SNAKE_STATE_DEAGGRO:
			if ((c->GetPosition() - anchorpoint).Length() < SNAKE_AGGRO_RADIUS &&
				c->GetColor() != GetColor() &&
				level->HasLineOfSight(position, c->GetPosition()))
			{
				state = SNAKE_STATE_AGGRO;
				max_speed = SNAKE_AGGRO_MAX_SPEED;
			}
			else if(anchorpoint.x - max_speed - 1 <= position.x && position.x <= anchorpoint.x + max_speed + 1
			&& anchorpoint.y - max_speed - 1 <= position.y && position.y <= anchorpoint.y + max_speed + 1)
				if(level->IsItDay())
				{
					state = SNAKE_STATE_DAY;
					day_wp_index = 0;
				}
				else
				{
					state = SNAKE_STATE_NIGHT;
					night_wp_index = 0;
				}
			break;
		default:
			assert(0);
	}

	Vec2f goal;

	//Set goal based on my state
	switch(state) {
		case SNAKE_STATE_AGGRO: goal = c->GetPosition(); break;
		case SNAKE_STATE_NIGHT: goal = night_waypoints.at(night_wp_index); break;
		case SNAKE_STATE_DAY: goal = day_waypoints.at(day_wp_index); break;
		case SNAKE_STATE_DEAGGRO: goal = anchorpoint; break;
		default: assert(0);
	}
	//Move

	velocity = level->GetShortestPath(GetPosition(), goal, max_speed);
	position += velocity;

	UpdateImage();
}

void Snake::Reset()
{
	this->GameObject::Reset();

	max_speed = SNAKE_DEFAULT_MAX_SPEED;
	radius = SNAKE_DEFAULT_RADIUS;

	day_wp_index = 0;
	night_wp_index = 0;
	anchorpoint = start_position;
	state = SNAKE_STATE_DAY;

	anim_index = 0;
	anim_timer = 0;

	UpdateImage();
}

void Snake::UpdateImage()
{
	if (velocity.x == 0 && velocity.y == 0) {
		image = still_image;
		anim_index = 0;
	} else {
		image = moving_image;

		if (anim_timer++ > SNAKE_ANIMATION_SPEED) {
			anim_timer = 0;
			anim_index = (anim_index + 1) % 4;
		}
	}
}

std::string Snake::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"snake\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";

	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														day_waypoints.begin(),
														day_waypoints.end());
	wptstring += LevelWriter::waypointsToXML(indent+"  ", position,
											 night_waypoints.begin(),
											 night_waypoints.end(),
											 false, true);
	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</creature>\n";

	return xml.str();
}

bool Snake::IsPatroller(){
	return true;
}

void Snake::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = day_waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = day_waypoints.begin();
		 i != day_waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);

	first = night_waypoints.front();
	for (std::vector<Vec2f>::iterator i = night_waypoints.begin();
		 i != night_waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void Snake::ResetWaypoints(Vec2f pos){
	day_waypoints.clear();
	day_waypoints.push_back(pos);
	night_waypoints.clear();
	night_waypoints.push_back(pos);
}
// vim: set ts=4 sts=4 sw=4 noet:
