/// \file SoundManager.cpp Source file for the SoundManager class

#include <cmath>

#include "SDL_mixer.h"

#include "SoundManager.hpp"

unsigned long SoundManager::currtime = 0;
Mix_Chunk *SoundManager::playing[SOUND_NUM_CHANNELS] = {NULL};

void SoundManager::Update()
{
	currtime++;
}

void SoundManager::PlayChunk(Mix_Chunk *sound, int vol, int loops,
							 int delay, int length, int musicvolume)
{
	if (!config->GetSoundEnabled() || sound == NULL)
		return;

	int channel = Mix_PlayChannelTimed(-1, sound, loops, length);
	if (channel == -1) {
		fprintf(stderr, "Error playing sound: %s\n", Mix_GetError());
		return;
	}

	playing[channel] = sound;
	Mix_Volume(channel, vol);
}

void SoundManager::StopChunk(Mix_Chunk *sound, int time)
{
	if (sound == NULL) {
		if (time == 0)
			Mix_HaltChannel(-1);
		else
			Mix_ExpireChannel(-1, time);
	}

	for (unsigned int i=0; i<SOUND_NUM_CHANNELS; i++)
		if (playing[i] == sound) {
			if (time == 0)
				Mix_HaltChannel(i);
			else
				Mix_ExpireChannel(i, time);
		}
}

bool SoundManager::IsChunkPlaying(const Mix_Chunk *sound)
{
	for (unsigned int i=0; i<SOUND_NUM_CHANNELS; i++)
		if (playing[i] == sound)
			return Mix_Playing(i) == 1;
	return false;
}

void SoundManager::PlayMusic(Mix_Chunk *music, int vol, int loops, int delay,
							 int length, int fadetime)
{
	if (!config->GetSoundEnabled() || music == NULL)
		return;

	int channel;
	if (fadetime <= 0)
		channel = Mix_PlayChannelTimed(-1, music, loops, length);
	else
		channel = Mix_FadeInChannelTimed(-1, music, loops, fadetime, length);

	if (channel == -1) {
		fprintf(stderr, "Error playing music: %s\n", Mix_GetError());
		return;
	}

	playing[channel] = music;
}

void SoundManager::PauseMusic(Mix_Chunk *music)
{
	for (unsigned int i=0; i<SOUND_NUM_CHANNELS; i++)
		if (playing[i] == music)
			Mix_Pause(i);
}

void SoundManager::ResumeMusic(Mix_Chunk *music)
{
	if (!config->GetSoundEnabled())
		return;

	for (unsigned int i=0; i<SOUND_NUM_CHANNELS; i++)
		if (playing[i] == music)
			Mix_Resume(i);
}

void SoundManager::StopMusic(Mix_Chunk *music, int time)
{
	StopChunk(music, time);
}

bool SoundManager::IsMusicPlaying(const Mix_Chunk *music)
{
	return IsChunkPlaying(music);
}

void SoundManager::FadeMusic(Mix_Chunk *music, float endvolume, float fadetime)
{
	// Not yet implemented
}

int SoundManager::GetMusicVolume(const Mix_Chunk *music)
{
	// Not yet implemented
	return 100;
}

int SoundManager::GetMasterVolume()
{
	// Not yet implemented
	return 100;
}

void SoundManager::SetMasterVolume(float volume)
{
	// Not yet implemented
}

// vim: set ts=4 sts=4 sw=4 noet:
