/// \file Switch.cpp Source file for the Switch class

#include <sstream>
#include "Switch.hpp"

SDL_Surface *Switch::switchImage;
Mix_Chunk *Switch::switchPressed;
Mix_Chunk *Switch::switchReleased;
unsigned int Switch::switchcount = 0;

Switch::Switch(Level *l, Vec2f p, bool t, int group) :
   	GameObject(l, p, SWITCH_DEFAULT_MAX_SPEED, SWITCH_DEFAULT_RADIUS),
   	toggle(t),
   	state(false),
   	triggered(false),
   	wastriggered(false),
   	switchgroup(group)
{
	if (switchcount++ == 0) {
		switchImage = LoadSprite(PKGDATADIR"Artwork/objects/switches/switch.png",
								 1,
								 level->GetTileWidth(),
								 level->GetTileHeight());
		switchPressed = LoadSound(PKGDATADIR"Sounds/SwitchDown.wav");
		switchReleased = LoadSound(PKGDATADIR"Sounds/SwitchUp.wav");
	}

	image = switchImage;
}

Switch::~Switch()
{
	if (--switchcount == 0) {
		SDL_FreeSurface(switchImage);
		Mix_FreeChunk(switchPressed);
		Mix_FreeChunk(switchReleased);
	}
}

void Switch::Run()
{
	if (disabled)
	   	return;

	if (triggered && !wastriggered) {
		// Something just stepped on the switch
		state = !state;
		for (unsigned int i=0; i<mySwitchables.size(); i++)
			mySwitchables[i]->SetState(!mySwitchables[i]->GetState());
		anim_index = 1;
		SoundManager::PlayChunk(switchPressed);
	} else if (!triggered && wastriggered) {
		// Something just got off of the switch
		if (!toggle) {
			state = !state;
			for (unsigned int i=0; i<mySwitchables.size(); i++)
				mySwitchables[i]->SetState(!mySwitchables[i]->GetState());
			SoundManager::PlayChunk(switchReleased);
		}
		anim_index = 0;
	}

	wastriggered = triggered;
	triggered = false; 
}

void Switch::Reset()
{
	this->GameObject::Reset();

	max_speed = SWITCH_DEFAULT_MAX_SPEED;
	radius = SWITCH_DEFAULT_RADIUS;

	state = false;
	triggered = false;
	wastriggered = false;
	anim_index = 0;

	//it's assumed the trigger object resets itself
}

std::string Switch::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<object x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"switch\"";

	if (switchgroup > 0)
		xml << " group=\"" << switchgroup << "\"";

	xml << "/>\n";
	return xml.str();
}
// vim: set ts=4 sts=4 sw=4 noet:
